@@ -67,6 +67,8 @@ later versions on an EPIA-M10000.
 
 #include <directfb.h>
 
+#include <fusion/shmalloc.h>
+
 #include <direct/messages.h>
 
 #include <core/coretypes.h>
@@ -76,9 +78,9 @@ later versions on an EPIA-M10000.
 #include <core/system.h>
 #include <core/screens.h>
 
-// System headers
+#include <fbdev/fb.h>
 
-#include <linux/fb.h>
+// System headers
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -410,13 +412,15 @@ static void driver_get_info(GraphicsDevice* device,
 static DFBResult driver_init_driver(GraphicsDevice* device,
                                     GraphicsDeviceFuncs* funcs,
                                     void* driver_data,
-                                    void* device_data)
+                                    void* device_data,
+                                    CoreDFB *core)
 {
      UcDriverData *ucdrv = (UcDriverData*) driver_data;
 
      //printf("Entering %s\n", __PRETTY_FUNCTION__);
 
      ucdrv->file = -1;
+     ucdrv->pool = dfb_core_shmpool( core );
 
      ucdrv->hwregs = dfb_gfxcard_map_mmio( device, 0, 0 );
      if (!ucdrv->hwregs) {
@@ -436,9 +440,10 @@ static DFBResult driver_init_driver(GraphicsDevice* device,
                return DFB_IO;
      }
 
-     ucdrv->fifo = uc_fifo_create(UC_FIFO_SIZE);
+     /* FIXME: this belongs to device_data! */
+     ucdrv->fifo = uc_fifo_create(ucdrv->pool, UC_FIFO_SIZE);
      if (!ucdrv->fifo)
-          return DFB_NOSYSTEMMEMORY;
+          return D_OOSHM();
 
      uc_after_set_var(driver_data, device_data);
 
@@ -532,7 +537,7 @@ static void driver_close_driver(GraphicsDevice* device, void* driver_data)
      UcDriverData* ucdrv = (UcDriverData*) driver_data;
 
      if (ucdrv->fifo)
-          uc_fifo_destroy( ucdrv->fifo );
+          uc_fifo_destroy( ucdrv->pool, ucdrv->fifo );
 
      if (ucdrv->file != -1)
           close( ucdrv->file );
