@@ -31,7 +31,7 @@
 #include <unistd.h>
 #include <stdlib.h>
 
-#include <linux/fb.h>
+#include <fbdev/fb.h>
 
 #include <sys/mman.h>
 #include <fcntl.h>
@@ -51,8 +51,6 @@
 #include <core/screens.h>
 #include <core/surfaces.h>
 
-#include <core/accelerators.h>
-
 #include <gfx/convert.h>
 
 #include <core/graphics_driver.h>
@@ -719,7 +717,8 @@ static DFBResult
 driver_init_driver( GraphicsDevice      *device,
                     GraphicsDeviceFuncs *funcs,
                     void                *driver_data,
-                    void                *device_data )
+                    void                *device_data,
+                    CoreDFB             *core )
 {
      CyberDriverData *cdrv = (CyberDriverData*) driver_data;
 
