@@ -21,7 +21,7 @@
    Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
 */
-#include <asm/types.h>
+#include <dfb_types.h>
 
 #include <stdio.h>
 #include <unistd.h>
@@ -39,8 +39,6 @@
 #include <core/coretypes.h>
 #include <core/screens.h>
 
-#include <core/accelerators.h>
-
 #include <core/state.h>
 #include <core/gfxcard.h>
 #include <core/surfaces.h>
@@ -591,6 +589,12 @@ typedef struct {
                     dda.mi--;              \
                }                           \
           }                                \
+          else {                           \
+               dda.mi   = 0;               \
+               dda.mf   = 0;               \
+               dda.xf   = 0;               \
+               dda._2dy = 0;               \
+          }                                \
      } while (0)
 
 
@@ -893,7 +897,8 @@ static DFBResult
 driver_init_driver( GraphicsDevice      *device,
                     GraphicsDeviceFuncs *funcs,
                     void                *driver_data,
-                    void                *device_data )
+                    void                *device_data,
+                    CoreDFB             *core )
 {
      DFBResult       ret;
      I810DriverData *idrv = driver_data;
