@@ -27,8 +27,7 @@
 #ifndef __I830_H__
 #define __I830_H__
 
-#include <asm/types.h>
-#include <sys/io.h>
+#include <dfb_types.h>
 #include <linux/agpgart.h>
 
 #include <core/gfxcard.h>
