@@ -108,8 +108,11 @@ ovInitLayer( CoreLayer                  *layer,
 
      /* set capabilities and type */
      description->caps = DLCAPS_SCREEN_LOCATION | DLCAPS_SURFACE |
-                         DLCAPS_SRC_COLORKEY | DLCAPS_DST_COLORKEY |
-                         DLCAPS_DEINTERLACING;
+                         DLCAPS_DST_COLORKEY | DLCAPS_DEINTERLACING;
+
+     if (mdev->chip >= CHIP_264VT3)
+          description->caps |= DLCAPS_SRC_COLORKEY;
+
      description->type = DLTF_VIDEO | DLTF_STILL_PICTURE;
 
      /* set name */
@@ -151,50 +154,46 @@ ovTestRegion( CoreLayer                  *layer,
      Mach64DriverData *mdrv = (Mach64DriverData*) driver_data;
      Mach64DeviceData *mdev = mdrv->device_data;
      CoreLayerRegionConfigFlags fail = 0;
-     int max_width, max_height;
+     int max_width, max_height = 1024;
 
      switch (mdev->chip) {
-          case CHIP_264VT: /* not verified */
+          case CHIP_264VT: /* 264VT2 verified */
           case CHIP_3D_RAGE: /* not verified */
                max_width = 384;
-               max_height = 1024;
                break;
           case CHIP_264VT3: /* not verified */
           case CHIP_3D_RAGE_II: /* not verified */
           case CHIP_3D_RAGE_IIPLUS:
-               max_width = 720;
-               max_height = 1024;
-               break;
           case CHIP_264VT4: /* not verified */
           case CHIP_3D_RAGE_IIC:
-               max_width = 720;
-               max_height = 2048;
-               break;
-          case CHIP_3D_RAGE_PRO: /* not verified */
-               max_width = 768;
-               max_height = 2048;
-               break;
-          case CHIP_3D_RAGE_LT_PRO:
-               max_width = 768;
-               max_height = 1024;
-               break;
           case CHIP_3D_RAGE_XLXC:
-               max_width = 720;
-               max_height = 2048;
-               break;
           case CHIP_3D_RAGE_MOBILITY:
                max_width = 720;
-               max_height = 1024;
+               break;
+          case CHIP_3D_RAGE_PRO: /* not verified */
+          case CHIP_3D_RAGE_LT_PRO:
+               max_width = 768;
                break;
           default:
                D_BUG( "unknown chip" );
                return DFB_UNSUPPORTED;
      }
 
+     if (config->options & DLOP_DEINTERLACING)
+          max_height = 2048;
+
      /* check for unsupported options */
      if (config->options & ~OV_SUPPORTED_OPTIONS)
           fail |= CLRCF_OPTIONS;
 
+     /*
+      * Video keying doesn't work the same way on 264VT2 as it does
+      * on later chips. If enabled the overlay goes completely black
+      * so clearly it does something but not what we want.
+      */
+     if (mdev->chip < CHIP_264VT3 && config->options & DLOP_SRC_COLORKEY)
+          fail |= CLRCF_OPTIONS;
+
      /* check pixel format */
      switch (config->format) {
           case DSPF_ARGB1555:
@@ -295,7 +294,14 @@ ovRemoveRegion( CoreLayer *layer,
      Mach64DeviceData *mdev = mdrv->device_data;
      volatile __u8    *mmio = mdrv->mmio_base;
 
-     mach64_waitfifo( mdrv, mdev, 1 );
+     mach64_waitfifo( mdrv, mdev, 2 );
+
+     /*
+      * On 264VT2 the keyer sometimes remains active
+      * even after the overlay has been disabled.
+      */
+     mach64_out32( mmio, OVERLAY_KEY_CNTL,
+                   VIDEO_KEY_FN_FALSE | GRAPHICS_KEY_FN_FALSE | OVERLAY_CMP_MIX_OR );
 
      mach64_out32( mmio, OVERLAY_SCALE_CNTL, 0 );
 
@@ -331,6 +337,9 @@ ovSetColorAdjustment( CoreLayer          *layer,
      Mach64DeviceData *mdev = mdrv->device_data;
      volatile __u8    *mmio = mdrv->mmio_base;
 
+     if (mdev->chip < CHIP_3D_RAGE_PRO)
+          return DFB_UNSUPPORTED;
+
      mach64_waitfifo( mdrv, mdev, 1 );
 
      mach64_out32( mmio, SCALER_COLOUR_CNTL,
@@ -388,11 +397,16 @@ static void ov_reset( Mach64DriverData *mdrv )
           mach64_out32( mmio, SCALER_COLOUR_CNTL, 0x00101000 );
      }
 
-     mach64_waitfifo( mdrv, mdev, 4 );
+     if (mdev->chip >= CHIP_264VT3) {
+          mach64_waitfifo( mdrv, mdev, 2 );
+
+          mach64_out32( mmio, OVERLAY_EXCLUSIVE_HORZ, 0 );
+          mach64_out32( mmio, OVERLAY_EXCLUSIVE_VERT, 0 );
+     }
+
+     mach64_waitfifo( mdrv, mdev, 2 );
 
      mach64_out32( mmio, OVERLAY_SCALE_CNTL, 0 );
-     mach64_out32( mmio, OVERLAY_EXCLUSIVE_HORZ, 0 );
-     mach64_out32( mmio, OVERLAY_EXCLUSIVE_VERT, 0 );
      mach64_out32( mmio, SCALER_TEST, 0 );
 }
 
@@ -695,7 +709,7 @@ static void ov_calc_opacity( Mach64DriverData       *mdrv,
                              Mach64OverlayLayerData *mov,
                              CoreLayerRegionConfig  *config )
 {
-     mov->regs.overlay_SCALE_CNTL = SCALE_PIX_EXPAND;
+     mov->regs.overlay_SCALE_CNTL = SCALE_PIX_EXPAND | SCALE_Y2R_TEMP;
 
      if (config->opacity && mov->visible)
           mov->regs.overlay_SCALE_CNTL |= OVERLAY_EN | SCALE_EN;
