@@ -72,11 +72,9 @@ void mach64_set_destination( Mach64DriverData *mdrv,
                break;
           default:
                D_BUG( "unexpected pixelformat!" );
-               break;
+               return;
      }
 
-     mdev->dst_key_mask = (1 << DFB_COLOR_BITS_PER_PIXEL( destination->format )) - 1;
-
      mach64_waitfifo( mdrv, mdev, 1 );
      mach64_out32( mmio, DST_OFF_PITCH, (buffer->video.offset/8) | ((pitch/8) << 22) );
 }
@@ -110,10 +108,15 @@ void mach64gt_set_destination( Mach64DriverData *mdrv,
                break;
           default:
                D_BUG( "unexpected pixelformat!" );
-               break;
+               return;
      }
 
-     mdev->dst_key_mask = (1 << DFB_COLOR_BITS_PER_PIXEL( destination->format )) - 1;
+     mdev->draw_blend &= ~DITHER_EN;
+     mdev->blit_blend &= ~DITHER_EN;
+     if (DFB_COLOR_BITS_PER_PIXEL( destination->format ) < 24) {
+          mdev->draw_blend |= DITHER_EN;
+          mdev->blit_blend |= DITHER_EN;
+     }
 
      mach64_waitfifo( mdrv, mdev, 1 );
      mach64_out32( mmio, DST_OFF_PITCH, (buffer->video.offset/8) | ((pitch/8) << 22) );
@@ -148,11 +151,9 @@ void mach64_set_source( Mach64DriverData *mdrv,
                break;
           default:
                D_BUG( "unexpected pixelformat!" );
-               break;
+               return;
      }
 
-     mdev->src_key_mask = (1 << DFB_COLOR_BITS_PER_PIXEL( source->format )) - 1;
-
      mach64_waitfifo( mdrv, mdev, 1 );
      mach64_out32( mmio, SRC_OFF_PITCH, (buffer->video.offset/8) | ((pitch/8) << 22) );
 
@@ -191,11 +192,9 @@ void mach64gt_set_source( Mach64DriverData *mdrv,
                break;
           default:
                D_BUG( "unexpected pixelformat!" );
-               break;
+               return;
      }
 
-     mdev->src_key_mask = (1 << DFB_COLOR_BITS_PER_PIXEL( source->format )) - 1;
-
      mach64_waitfifo( mdrv, mdev, 1 );
      mach64_out32( mmio, SRC_OFF_PITCH, (buffer->video.offset/8) | ((pitch/8) << 22) );
 
@@ -236,11 +235,12 @@ void mach64gt_set_source_scale( Mach64DriverData *mdrv,
                break;
           default:
                D_BUG( "unexpected pixelformat!" );
-               break;
+               return;
      }
 
-     /* FIXME: Scaler/texture color key is RGB24 on < 3D Rage Pro. */
-     mdev->src_key_mask = (1 << DFB_COLOR_BITS_PER_PIXEL( source->format )) - 1;
+     mdev->blit_blend &= ~SCALE_PIX_EXPAND;
+     if (DFB_COLOR_BITS_PER_PIXEL( source->format ) < 24)
+          mdev->blit_blend |= SCALE_PIX_EXPAND;
 
      mdev->field = source->field;
      if (mdev->blit_deinterlace) {
@@ -293,53 +293,60 @@ void mach64_set_color( Mach64DriverData *mdrv,
                        Mach64DeviceData *mdev,
                        CardState        *state )
 {
-     volatile __u8 *mmio = mdrv->mmio_base;
-     __u32          color = 0;
+     volatile __u8 *mmio  = mdrv->mmio_base;
+     DFBColor       color = state->color;
+     __u32          clr;
 
      if (MACH64_IS_VALID( m_color ))
           return;
 
+     if (state->drawingflags & DSDRAW_SRC_PREMULTIPLY) {
+          color.r = (color.r * (color.a + 1)) >> 8;
+          color.g = (color.g * (color.a + 1)) >> 8;
+          color.b = (color.b * (color.a + 1)) >> 8;
+     }
+
      switch (state->destination->format) {
           case DSPF_RGB332:
-               color = PIXEL_RGB332( state->color.r,
-                                     state->color.g,
-                                     state->color.b );
+               clr = PIXEL_RGB332( color.r,
+                                   color.g,
+                                   color.b );
                break;
           case DSPF_ARGB1555:
-               color = PIXEL_ARGB1555( state->color.a,
-                                       state->color.r,
-                                       state->color.g,
-                                       state->color.b );
+               clr = PIXEL_ARGB1555( color.a,
+                                     color.r,
+                                     color.g,
+                                     color.b );
                break;
           case DSPF_ARGB4444:
-               color = PIXEL_ARGB4444( state->color.a,
-                                       state->color.r,
-                                       state->color.g,
-                                       state->color.b );
+               clr = PIXEL_ARGB4444( color.a,
+                                     color.r,
+                                     color.g,
+                                     color.b );
                break;
           case DSPF_RGB16:
-               color = PIXEL_RGB16( state->color.r,
-                                    state->color.g,
-                                    state->color.b );
+               clr = PIXEL_RGB16( color.r,
+                                  color.g,
+                                  color.b );
                break;
           case DSPF_RGB32:
-               color = PIXEL_RGB32( state->color.r,
-                                    state->color.g,
-                                    state->color.b );
+               clr = PIXEL_RGB32( color.r,
+                                  color.g,
+                                  color.b );
                break;
           case DSPF_ARGB:
-               color = PIXEL_ARGB( state->color.a,
-                                   state->color.r,
-                                   state->color.g,
-                                   state->color.b );
+               clr = PIXEL_ARGB( color.a,
+                                 color.r,
+                                 color.g,
+                                 color.b );
                break;
           default:
                D_BUG( "unexpected pixelformat!" );
-               break;
+               return;
      }
 
      mach64_waitfifo( mdrv, mdev, 1 );
-     mach64_out32( mmio, DP_FRGD_CLR, color );
+     mach64_out32( mmio, DP_FRGD_CLR, clr );
 
      MACH64_VALIDATE( m_color );
 }
@@ -348,25 +355,66 @@ void mach64_set_color_3d( Mach64DriverData *mdrv,
                           Mach64DeviceData *mdev,
                           CardState        *state )
 {
-     volatile __u8 *mmio = mdrv->mmio_base;
+     volatile __u8 *mmio  = mdrv->mmio_base;
+     DFBColor       color = state->color;
 
      if (MACH64_IS_VALID( m_color_3d ))
           return;
 
+     if (state->drawingflags & DSDRAW_SRC_PREMULTIPLY) {
+          color.r = (color.r * (color.a + 1)) >> 8;
+          color.g = (color.g * (color.a + 1)) >> 8;
+          color.b = (color.b * (color.a + 1)) >> 8;
+     }
+
      /* Some 3D color registers scaler registers are shared. */
      mach64_waitfifo( mdrv, mdev, 7 );
      mach64_out32( mmio, RED_X_INC, 0 );
-     mach64_out32( mmio, RED_START, state->color.r << 16 );
+     mach64_out32( mmio, RED_START, color.r << 16 );
      mach64_out32( mmio, GREEN_X_INC, 0 );
-     mach64_out32( mmio, GREEN_START, state->color.g << 16 );
+     mach64_out32( mmio, GREEN_START, color.g << 16 );
      mach64_out32( mmio, BLUE_X_INC, 0 );
-     mach64_out32( mmio, BLUE_START, state->color.b << 16 );
-     mach64_out32( mmio, ALPHA_START, state->color.a << 16 );
+     mach64_out32( mmio, BLUE_START, color.b << 16 );
+     mach64_out32( mmio, ALPHA_START, color.a << 16 );
 
-     MACH64_INVALIDATE( m_blit_blend );
+     MACH64_INVALIDATE( m_color_tex | m_blit_blend );
      MACH64_VALIDATE( m_color_3d );
 }
 
+void mach64_set_color_tex( Mach64DriverData *mdrv,
+                           Mach64DeviceData *mdev,
+                           CardState        *state )
+{
+     volatile __u8 *mmio  = mdrv->mmio_base;
+     DFBColor       color = state->color;
+
+     if (MACH64_IS_VALID( m_color_tex ))
+          return;
+
+     if (state->blittingflags & DSBLIT_SRC_PREMULTCOLOR) {
+          if (state->blittingflags & DSBLIT_COLORIZE) {
+               color.r = (color.r * (color.a + 1)) >> 8;
+               color.g = (color.g * (color.a + 1)) >> 8;
+               color.b = (color.b * (color.a + 1)) >> 8;
+          } else {
+               color.r = color.g = color.b = color.a;
+          }
+     }
+
+     /* Some 3D color registers scaler registers are shared. */
+     mach64_waitfifo( mdrv, mdev, 7 );
+     mach64_out32( mmio, RED_X_INC, 0 );
+     mach64_out32( mmio, RED_START, color.r << 16 );
+     mach64_out32( mmio, GREEN_X_INC, 0 );
+     mach64_out32( mmio, GREEN_START, color.g << 16 );
+     mach64_out32( mmio, BLUE_X_INC, 0 );
+     mach64_out32( mmio, BLUE_START, color.b << 16 );
+     mach64_out32( mmio, ALPHA_START, color.a << 16 );
+
+     MACH64_INVALIDATE( m_color_3d | m_blit_blend );
+     MACH64_VALIDATE( m_color_tex );
+}
+
 void mach64_set_src_colorkey( Mach64DriverData *mdrv,
                               Mach64DeviceData *mdev,
                               CardState        *state )
@@ -377,7 +425,8 @@ void mach64_set_src_colorkey( Mach64DriverData *mdrv,
           return;
 
      mach64_waitfifo( mdrv, mdev, 3 );
-     mach64_out32( mmio, CLR_CMP_MSK, mdev->src_key_mask );
+     mach64_out32( mmio, CLR_CMP_MSK,
+                   (1 << DFB_COLOR_BITS_PER_PIXEL( state->source->format )) - 1 );
      mach64_out32( mmio, CLR_CMP_CLR, state->src_colorkey );
      mach64_out32( mmio, CLR_CMP_CNTL, CLR_CMP_FN_EQUAL | CLR_CMP_SRC_2D );
 
@@ -390,14 +439,54 @@ void mach64_set_src_colorkey_scale( Mach64DriverData *mdrv,
                                     CardState        *state )
 {
      volatile __u8 *mmio = mdrv->mmio_base;
+     __u32          clr, msk;
 
      if (MACH64_IS_VALID( m_srckey_scale ))
           return;
 
-     /* FIXME: Scaler/texture color key is RGB24 on < 3D Rage Pro. */
+     if (mdev->chip < CHIP_3D_RAGE_PRO) {
+          switch (state->source->format) {
+               case DSPF_RGB332:
+                    clr = ((state->src_colorkey & 0xE0) << 16) |
+                          ((state->src_colorkey & 0x1C) << 11) |
+                          ((state->src_colorkey & 0x03) <<  6);
+                    msk = 0xE0E0C0;
+                    break;
+               case DSPF_ARGB4444:
+                    clr = ((state->src_colorkey & 0x0F00) << 12) |
+                          ((state->src_colorkey & 0x00F0) <<  8) |
+                          ((state->src_colorkey & 0x000F) <<  4);
+                    msk = 0xF0F0F0;
+                    break;
+               case DSPF_ARGB1555:
+                    clr = ((state->src_colorkey & 0x7C00) << 9) |
+                          ((state->src_colorkey & 0x03E0) << 6) |
+                          ((state->src_colorkey & 0x001F) << 3);
+                    msk = 0xF8F8F8;
+                    break;
+               case DSPF_RGB16:
+                    clr = ((state->src_colorkey & 0xF800) << 8) |
+                          ((state->src_colorkey & 0x07E0) << 5) |
+                          ((state->src_colorkey & 0x001F) << 3);
+                    msk = 0xF8FCF8;
+                    break;
+               case DSPF_RGB32:
+               case DSPF_ARGB:
+                    clr = state->src_colorkey;
+                    msk = 0xFFFFFF;
+                    break;
+               default:
+                    D_BUG( "unexpected pixelformat!" );
+                    return;
+          }
+     } else {
+          clr = state->src_colorkey;
+          msk = (1 << DFB_COLOR_BITS_PER_PIXEL( state->source->format )) - 1;
+     }
+
      mach64_waitfifo( mdrv, mdev, 3 );
-     mach64_out32( mmio, CLR_CMP_MSK, mdev->src_key_mask );
-     mach64_out32( mmio, CLR_CMP_CLR, state->src_colorkey );
+     mach64_out32( mmio, CLR_CMP_MSK, msk );
+     mach64_out32( mmio, CLR_CMP_CLR, clr );
      mach64_out32( mmio, CLR_CMP_CNTL, CLR_CMP_FN_EQUAL | CLR_CMP_SRC_SCALE );
 
      MACH64_VALIDATE( m_srckey_scale );
@@ -414,7 +503,8 @@ void mach64_set_dst_colorkey( Mach64DriverData *mdrv,
           return;
 
      mach64_waitfifo( mdrv, mdev, 3 );
-     mach64_out32( mmio, CLR_CMP_MSK, mdev->dst_key_mask );
+     mach64_out32( mmio, CLR_CMP_MSK,
+                   (1 << DFB_COLOR_BITS_PER_PIXEL( state->destination->format )) - 1 );
      mach64_out32( mmio, CLR_CMP_CLR, state->dst_colorkey );
      mach64_out32( mmio, CLR_CMP_CNTL, CLR_CMP_FN_NOT_EQUAL | CLR_CMP_SRC_DEST );
 
@@ -474,10 +564,10 @@ void mach64_set_draw_blend( Mach64DriverData *mdrv,
      if (MACH64_IS_VALID( m_draw_blend ))
           return;
 
-     mdev->draw_blend = SCALE_PIX_EXPAND | DITHER_EN |
-                        ALPHA_FOG_EN_ALPHA |
-                        mach64SourceBlend[state->src_blend - 1] |
-                        mach64DestBlend  [state->dst_blend - 1];
+     mdev->draw_blend &= DITHER_EN;
+     mdev->draw_blend |= ALPHA_FOG_EN_ALPHA |
+                         mach64SourceBlend[state->src_blend - 1] |
+                         mach64DestBlend  [state->dst_blend - 1];
 
      if (mdev->chip >= CHIP_3D_RAGE_PRO) {
           /* FIXME: This is wrong. */
@@ -497,7 +587,7 @@ void mach64_set_blit_blend( Mach64DriverData *mdrv,
      if (MACH64_IS_VALID( m_blit_blend ))
           return;
 
-     mdev->blit_blend = SCALE_PIX_EXPAND | DITHER_EN;
+     mdev->blit_blend &= SCALE_PIX_EXPAND | DITHER_EN;
 
      if (state->blittingflags & (DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA)) {
           mdev->blit_blend |= ALPHA_FOG_EN_ALPHA |
@@ -510,7 +600,7 @@ void mach64_set_blit_blend( Mach64DriverData *mdrv,
                } else {
                     mach64_waitfifo( mdrv, mdev, 1 );
                     mach64_out32( mmio, ALPHA_START, 0xFF << 16 );
-                    MACH64_INVALIDATE( m_color_3d );
+                    MACH64_INVALIDATE( m_color_3d | m_color_tex );
                }
           }
 
@@ -533,7 +623,7 @@ void mach64_set_blit_blend( Mach64DriverData *mdrv,
           }
      }
 
-     if (state->blittingflags & DSBLIT_COLORIZE)
+     if (state->blittingflags & (DSBLIT_COLORIZE | DSBLIT_SRC_PREMULTCOLOR))
           mdev->blit_blend |= TEX_LIGHT_FCN_MODULATE;
 
      MACH64_VALIDATE( m_blit_blend );
