@@ -25,6 +25,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+#include <config.h>
+
 #include <stdio.h>
 #include <unistd.h>
 #include <stdlib.h>
@@ -33,7 +35,7 @@
 #include <fcntl.h>
 #include <sys/ioctl.h>
 
-#include <linux/fb.h>
+#include <fbdev/fb.h>
 
 #include <math.h>
 
@@ -50,8 +52,6 @@
 #include <core/surfaces.h>
 #include <core/palette.h>
 
-#include <core/accelerators.h>
-
 #include <gfx/util.h>
 
 #include <misc/conf.h>
@@ -108,7 +108,7 @@ static bool matroxStretchBlit_422( void *drv, void *dev,
 
 /* Millennium */
 
-#define MATROX_2064W_DRAWING_FLAGS          (DSDRAW_NOFX)
+#define MATROX_2064W_DRAWING_FLAGS          (DSDRAW_SRC_PREMULTIPLY)
 
 #define MATROX_2064W_BLITTING_FLAGS         (DSBLIT_NOFX)
 
@@ -122,7 +122,7 @@ static bool matroxStretchBlit_422( void *drv, void *dev,
 
 /* Old cards (Mystique, Millennium II) */
 
-#define MATROX_OLD_DRAWING_FLAGS            (DSDRAW_NOFX)
+#define MATROX_OLD_DRAWING_FLAGS            (DSDRAW_SRC_PREMULTIPLY)
 
 #define MATROX_OLD_BLITTING_FLAGS           (DSBLIT_SRC_COLORKEY)
 
@@ -136,7 +136,7 @@ static bool matroxStretchBlit_422( void *drv, void *dev,
 
 /* G100 */
 
-#define MATROX_G100_DRAWING_FLAGS           (DSDRAW_NOFX)
+#define MATROX_G100_DRAWING_FLAGS           (DSDRAW_SRC_PREMULTIPLY)
 
 #define MATROX_G100_BLITTING_FLAGS          (DSBLIT_SRC_COLORKEY | \
                                            /*DSBLIT_BLEND_ALPHACHANNEL |*/ \
@@ -155,13 +155,15 @@ static bool matroxStretchBlit_422( void *drv, void *dev,
 
 /* G200/G400 */
 
-#define MATROX_G200G400_DRAWING_FLAGS       (DSDRAW_BLEND)
+#define MATROX_G200G400_DRAWING_FLAGS       (DSDRAW_BLEND | \
+                                             DSDRAW_SRC_PREMULTIPLY)
 
 #define MATROX_G200G400_BLITTING_FLAGS      (DSBLIT_SRC_COLORKEY | \
                                              DSBLIT_BLEND_ALPHACHANNEL | \
                                              DSBLIT_BLEND_COLORALPHA | \
                                              DSBLIT_COLORIZE | \
                                              DSBLIT_DEINTERLACE | \
+                                             DSBLIT_SRC_PREMULTIPLY | \
                                              DSBLIT_SRC_PREMULTCOLOR)
 
 #define MATROX_G200G400_DRAWING_FUNCTIONS   (DFXL_FILLRECTANGLE | \
@@ -179,6 +181,7 @@ static bool matroxStretchBlit_422( void *drv, void *dev,
                                 DSBLIT_BLEND_COLORALPHA   |              \
                                 DSBLIT_COLORIZE           |              \
                                 DSBLIT_DEINTERLACE        |              \
+                                DSBLIT_SRC_PREMULTIPLY    |              \
                                 DSBLIT_SRC_PREMULTCOLOR)       ||        \
       ((state)->destination->format != (state)->source->format &&        \
        (state)->destination->format != DSPF_I420               &&        \
@@ -211,13 +214,21 @@ matroxEngineReset( void *drv, void *dev )
      mga_out32( mmio, 0, DR15 );
      mga_out32( mmio, 0, BCOL );
 
-     mga_waitfifo( mdrv, mdev, 6 );
+     mga_waitfifo( mdrv, mdev, 5 );
      mga_out32( mmio, 0, TMR1 );
      mga_out32( mmio, 0, TMR2 );
      mga_out32( mmio, 0, TMR4 );
      mga_out32( mmio, 0, TMR5 );
      mga_out32( mmio, 0x100000, TMR8 );
-     mga_out32( mmio, 0xFFFFFFFF, PLNWT );
+
+     /*
+      * Plane write mask is not supported on G100 with SDRAM.
+      * The chip goes crazy if PLNWT is written.
+      */
+     if (mdrv->accelerator != FB_ACCEL_MATROX_MGAG100) {
+          mga_waitfifo( mdrv, mdev, 1 );
+          mga_out32( mmio, 0xFFFFFFFF, PLNWT );
+     }
 }
 
 static void
@@ -239,6 +250,14 @@ matroxFlushTextureCache( void *drv, void *dev )
      mga_out32( mdrv->mmio_base, 0, TEXORG1 );
 }
 
+static void
+matroxFlushReadCache( void *drv, void *dev )
+{
+     MatroxDriverData *mdrv = (MatroxDriverData*) drv;
+
+     mga_out8( mdrv->mmio_base, 0, CACHEFLUSH );
+}
+
 static bool
 matrox_check_blend( MatroxDeviceData *mdev,
                     CardState        *state )
@@ -482,10 +501,19 @@ matroxG200CheckState( void *drv, void *dev,
           if (state->blittingflags & ~MATROX_G200G400_BLITTING_FLAGS)
                return;
 
-          if (state->blittingflags & (DSBLIT_BLEND_ALPHACHANNEL |
-                                      DSBLIT_BLEND_COLORALPHA) &&
-              !matrox_check_blend( mdev, state ))
-               return;
+          if (state->blittingflags & (DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA)) {
+               if (!matrox_check_blend( mdev, state ))
+                    return;
+
+               if (state->blittingflags & DSBLIT_SRC_PREMULTIPLY &&
+                   (state->src_blend != DSBF_ONE ||
+                    (state->dst_blend != DSBF_INVSRCALPHA &&
+                     state->dst_blend != DSBF_INVSRCCOLOR)))
+                    return;
+          } else {
+               if (state->blittingflags & DSBLIT_SRC_PREMULTIPLY)
+                    return;
+          }
 
           if (use_tmu) {
                if (state->source->width < 8 ||
@@ -591,10 +619,19 @@ matroxG400CheckState( void *drv, void *dev,
           if (state->blittingflags & ~MATROX_G200G400_BLITTING_FLAGS)
                return;
 
-          if (state->blittingflags & (DSBLIT_BLEND_ALPHACHANNEL |
-                                      DSBLIT_BLEND_COLORALPHA) &&
-              !matrox_check_blend( mdev, state ))
-               return;
+          if (state->blittingflags & (DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA)) {
+               if (!matrox_check_blend( mdev, state ))
+                    return;
+
+               if (state->blittingflags & DSBLIT_SRC_PREMULTIPLY &&
+                   (state->src_blend != DSBF_ONE ||
+                    (state->dst_blend != DSBF_INVSRCALPHA &&
+                     state->dst_blend != DSBF_INVSRCCOLOR)))
+                    return;
+          } else {
+               if (state->blittingflags & DSBLIT_SRC_PREMULTIPLY)
+                    return;
+          }
 
           if (use_tmu) {
                if (state->source->width < 8 ||
@@ -634,30 +671,25 @@ matroxSetState( void *drv, void *dev,
      else if (state->modified) {
           if (state->modified & SMF_COLOR)
                MGA_INVALIDATE( m_drawColor | m_blitColor | m_color );
-          else if (state->modified & SMF_DESTINATION)
-               MGA_INVALIDATE( m_color );
+
+          if (state->modified & SMF_DESTINATION)
+               MGA_INVALIDATE( m_color | m_Source | m_source );
 
           if (state->modified & SMF_SOURCE)
                MGA_INVALIDATE( m_Source | m_source | m_SrcKey | m_srckey | m_blitBlend );
           else if (state->modified & SMF_SRC_COLORKEY)
                MGA_INVALIDATE( m_SrcKey | m_srckey );
 
+          if (state->modified & SMF_DRAWING_FLAGS)
+               MGA_INVALIDATE( m_drawColor | m_color );
+
           if (state->modified & SMF_BLITTING_FLAGS)
-               MGA_INVALIDATE( m_Source | m_blitBlend | m_blitColor );
+               MGA_INVALIDATE( m_Source | m_SrcKey | m_blitBlend | m_blitColor );
 
           if (state->modified & (SMF_DST_BLEND | SMF_SRC_BLEND))
                MGA_INVALIDATE( m_blitBlend | m_drawBlend );
      }
 
-
-     if (state->modified & SMF_DESTINATION) {
-          matrox_set_destination( mdrv, mdev, state->destination );
-
-          /* On old cards the clip depends on the destination's pixel offset */
-          if (mdev->old_matrox)
-               state->modified |= SMF_CLIP;
-     }
-
      switch (accel) {
           case DFXL_FILLRECTANGLE:
           case DFXL_DRAWRECTANGLE:
@@ -737,8 +769,7 @@ matroxSetState( void *drv, void *dev,
                     matrox_validate_blitBlend( mdrv, mdev, state );
                     matrox_validate_Source( mdrv, mdev, state );
 
-                    if (mdev->blit_src_colorkey)
-                         matrox_validate_SrcKey( mdrv, mdev, state );
+                    matrox_validate_SrcKey( mdrv, mdev, state );
                }
                else {
                     switch (state->destination->format) {
@@ -772,9 +803,18 @@ matroxSetState( void *drv, void *dev,
                break;
      }
 
+     if (state->modified & SMF_DESTINATION) {
+          matrox_set_destination( mdrv, mdev, state->destination );
+          state->modified |= SMF_CLIP;
+     }
+
      if (state->modified & SMF_CLIP) {
-          matrox_set_clip( mdrv, mdev, &state->clip );
           mdev->clip = state->clip;
+          if (state->destination->format == DSPF_YUY2 || state->destination->format == DSPF_UYVY) {
+               mdev->clip.x1 /= 2;
+               mdev->clip.x2 /= 2;
+          }
+          matrox_set_clip( mdrv, mdev, &mdev->clip );
      }
 
      state->modified = 0;
@@ -1702,7 +1742,15 @@ static __u32 pci_config_in32( unsigned int bus,
      }
 
      close( fd );
+
+#ifdef WORDS_BIGENDIAN
+     return ((val & 0xff000000) >> 24) |
+            ((val & 0x00ff0000) >>  8) |
+            ((val & 0x0000ff00) <<  8) |
+            ((val & 0x000000ff) << 24);
+#else
      return val;
+#endif
 }
 
 static DFBResult matrox_find_pci_device( MatroxDeviceData *mdev,
@@ -1737,8 +1785,8 @@ static DFBResult matrox_find_pci_device( MatroxDeviceData *mdev,
           addr1 &= ~0xFUL;
 
           switch (device) {
-          case PCI_DEVICE_ID_MATROX_G550:
-          case PCI_DEVICE_ID_MATROX_G400:
+          case PCI_DEVICE_ID_MATROX_G550_AGP:
+          case PCI_DEVICE_ID_MATROX_G400_AGP:
                if (addr0 == (mdev->fb.physical & ~0x1FFFFFF)) {
                     fclose( file );
                     return DFB_OK;
@@ -1747,25 +1795,26 @@ static DFBResult matrox_find_pci_device( MatroxDeviceData *mdev,
 
           case PCI_DEVICE_ID_MATROX_G200_PCI:
           case PCI_DEVICE_ID_MATROX_G200_AGP:
-          case PCI_DEVICE_ID_MATROX_G100_MM:
+          case PCI_DEVICE_ID_MATROX_G100_PCI:
           case PCI_DEVICE_ID_MATROX_G100_AGP:
-          case PCI_DEVICE_ID_MATROX_MIL_2:
-          case PCI_DEVICE_ID_MATROX_MIL_2_AGP:
+          case PCI_DEVICE_ID_MATROX_2164W_PCI:
+          case PCI_DEVICE_ID_MATROX_2164W_AGP:
                if (addr0 == mdev->fb.physical) {
                     fclose( file );
                     return DFB_OK;
                }
                break;
 
-          case PCI_DEVICE_ID_MATROX_MYS:
-               if ((pci_config_in32( *bus, *slot, *func, 0x08 ) & 0xFF) >= 0x02) {
-                    /* Mystique 220 */
+          case PCI_DEVICE_ID_MATROX_1064SG_PCI:
+          case PCI_DEVICE_ID_MATROX_1064SG_AGP:
+               if ((pci_config_in32( *bus, *slot, *func, 0x08 ) & 0xFF) > 0x02) {
+                    /* Mystique 220 (1164SG) */
                     if (addr0 == mdev->fb.physical) {
                          fclose( file );
                          return DFB_OK;
                     }
                } else {
-                    /* Mystique */
+                    /* Mystique (1064SG) */
                     if (addr1 == mdev->fb.physical) {
                          fclose( file );
                          return DFB_OK;
@@ -1773,7 +1822,7 @@ static DFBResult matrox_find_pci_device( MatroxDeviceData *mdev,
                }
                break;
 
-          case PCI_DEVICE_ID_MATROX_MIL:
+          case PCI_DEVICE_ID_MATROX_2064W_PCI:
                if (addr1 == mdev->fb.physical) {
                     fclose( file );
                     return DFB_OK;
@@ -1794,13 +1843,13 @@ static int
 driver_probe( GraphicsDevice *device )
 {
      switch (dfb_gfxcard_get_accelerator( device )) {
-          case FB_ACCEL_MATROX_MGA2064W:     /* Matrox MGA2064W (Millennium)   */
-          case FB_ACCEL_MATROX_MGA1064SG:    /* Matrox MGA1064SG (Mystique)   */
-          case FB_ACCEL_MATROX_MGA2164W:     /* Matrox MGA2164W (Millennium II)*/
-          case FB_ACCEL_MATROX_MGA2164W_AGP: /* Matrox MGA2164W (Millennium II)*/
-          case FB_ACCEL_MATROX_MGAG100:      /* Matrox G100                   */
-          case FB_ACCEL_MATROX_MGAG200:      /* Matrox G200 (Myst, Mill, ...) */
-          case FB_ACCEL_MATROX_MGAG400:      /* Matrox G400                   */
+          case FB_ACCEL_MATROX_MGA2064W:     /* Matrox 2064W (Millennium)       */
+          case FB_ACCEL_MATROX_MGA1064SG:    /* Matrox 1064SG/1164SG (Mystique) */
+          case FB_ACCEL_MATROX_MGA2164W:     /* Matrox 2164W (Millennium II)    */
+          case FB_ACCEL_MATROX_MGA2164W_AGP: /* Matrox 2164W (Millennium II)    */
+          case FB_ACCEL_MATROX_MGAG100:      /* Matrox G100                     */
+          case FB_ACCEL_MATROX_MGAG200:      /* Matrox G200                     */
+          case FB_ACCEL_MATROX_MGAG400:      /* Matrox G400/G450/G550           */
                return 1;
      }
 
@@ -1831,7 +1880,8 @@ static DFBResult
 driver_init_driver( GraphicsDevice      *device,
                     GraphicsDeviceFuncs *funcs,
                     void                *driver_data,
-                    void                *device_data )
+                    void                *device_data,
+                    CoreDFB             *core )
 {
      MatroxDriverData *mdrv = driver_data;
 
@@ -1873,6 +1923,7 @@ driver_init_driver( GraphicsDevice      *device,
      funcs->EngineReset       = matroxEngineReset;
      funcs->EngineSync        = matroxEngineSync;
      funcs->FlushTextureCache = matroxFlushTextureCache;
+     funcs->FlushReadCache    = matroxFlushReadCache;
 
      funcs->DrawRectangle     = matroxDrawRectangle;
      funcs->DrawLine          = matroxDrawLine;
@@ -1923,8 +1974,7 @@ driver_init_device( GraphicsDevice     *device,
                if ((ret = matrox_find_pci_device( mdev, &bus, &slot, &func )))
                     return ret;
 
-               g550  = ((pci_config_in32( bus, slot, func, 0x00 ) >> 16) ==
-                        PCI_DEVICE_ID_MATROX_G550);
+               g550  = ((pci_config_in32( bus, slot, func, 0x00 ) >> 16) == PCI_DEVICE_ID_MATROX_G550_AGP);
                g450  = ((pci_config_in32( bus, slot, func, 0x08 ) & 0xFF) >= 0x80);
                sgram = ((pci_config_in32( bus, slot, func, 0x40 ) & 0x4000) == 0x4000);
                snprintf( device_info->name,
@@ -1945,7 +1995,7 @@ driver_init_device( GraphicsDevice     *device,
                break;
           case FB_ACCEL_MATROX_MGAG100:
                mdev->old_matrox = true;
-               sgram            = true;
+               sgram            = false; /* FIXME: can we detect this? */
                snprintf( device_info->name,
                          DFB_GRAPHICS_DEVICE_INFO_NAME_LENGTH, "G100" );
                break;
@@ -1953,7 +2003,7 @@ driver_init_device( GraphicsDevice     *device,
                mdev->old_matrox = true;
                sgram            = true;
                snprintf( device_info->name,
-                         DFB_GRAPHICS_DEVICE_INFO_NAME_LENGTH, "Millennium I" );
+                         DFB_GRAPHICS_DEVICE_INFO_NAME_LENGTH, "Millennium (2064W)" );
                break;
           case FB_ACCEL_MATROX_MGA1064SG:
                if ((ret = matrox_find_pci_device( mdev, &bus, &slot, &func )))
@@ -1962,14 +2012,16 @@ driver_init_device( GraphicsDevice     *device,
                mdev->old_matrox = true;
                sgram            = ((pci_config_in32( bus, slot, func, 0x40 ) & 0x4000) == 0x4000);
                snprintf( device_info->name,
-                         DFB_GRAPHICS_DEVICE_INFO_NAME_LENGTH, "Mystique" );
+                         DFB_GRAPHICS_DEVICE_INFO_NAME_LENGTH, "%s",
+                         ((pci_config_in32( bus, slot, func, 0x08 ) & 0xFF) > 0x02) ?
+                         "Mystique 220 (1164SG)" : "Mystique (1064SG)" );
                break;
           case FB_ACCEL_MATROX_MGA2164W:
           case FB_ACCEL_MATROX_MGA2164W_AGP:
                mdev->old_matrox = true;
                sgram            = true;
                snprintf( device_info->name,
-                         DFB_GRAPHICS_DEVICE_INFO_NAME_LENGTH, "Millennium II" );
+                         DFB_GRAPHICS_DEVICE_INFO_NAME_LENGTH, "Millennium II (2164W)" );
                break;
      }
 
