@@ -34,16 +34,17 @@
 #include <core/screens.h>
 
 #define PCI_VENDOR_ID_MATROX            0x102B
-#define PCI_DEVICE_ID_MATROX_MIL        0x0519
-#define PCI_DEVICE_ID_MATROX_MYS        0x051A
-#define PCI_DEVICE_ID_MATROX_MIL_2      0x051b
-#define PCI_DEVICE_ID_MATROX_MIL_2_AGP  0x051f
-#define PCI_DEVICE_ID_MATROX_G100_MM    0x1000
+#define PCI_DEVICE_ID_MATROX_2064W_PCI  0x0519
+#define PCI_DEVICE_ID_MATROX_1064SG_PCI 0x051A
+#define PCI_DEVICE_ID_MATROX_2164W_PCI  0x051B
+#define PCI_DEVICE_ID_MATROX_1064SG_AGP 0x051E
+#define PCI_DEVICE_ID_MATROX_2164W_AGP  0x051F
+#define PCI_DEVICE_ID_MATROX_G100_PCI   0x1000
 #define PCI_DEVICE_ID_MATROX_G100_AGP   0x1001
 #define PCI_DEVICE_ID_MATROX_G200_PCI   0x0520
 #define PCI_DEVICE_ID_MATROX_G200_AGP   0x0521
-#define PCI_DEVICE_ID_MATROX_G400       0x0525
-#define PCI_DEVICE_ID_MATROX_G550       0x2527
+#define PCI_DEVICE_ID_MATROX_G400_AGP   0x0525
+#define PCI_DEVICE_ID_MATROX_G550_AGP   0x2527
 
 typedef enum {
      m_Source       = 0x0001,
@@ -114,8 +115,6 @@ typedef struct {
      } fb;
      unsigned int tlut_offset;
      CorePalette *rgb332_palette;
-
-     DFBSurfacePixelFormat dst_format;
 } MatroxDeviceData;
 
 typedef struct {
