@@ -33,7 +33,7 @@
 #include <fcntl.h>
 #include <sys/ioctl.h>
 
-#include <linux/fb.h>
+#include <fbdev/fb.h>
 
 #include <directfb.h>
 
@@ -41,7 +41,6 @@
 
 #include <fusion/shmalloc.h>
 
-#include <core/accelerators.h>
 #include <core/coredefs.h>
 #include <core/coretypes.h>
 #include <core/gfxcard.h>
@@ -54,6 +53,7 @@
 
 #include <direct/mem.h>
 
+#include <misc/util.h>
 
 #include "regs.h"
 #include "mmio.h"
@@ -101,7 +101,8 @@ typedef struct {
 static void bes_set_regs( MatroxDriverData *mdrv, MatroxBesLayerData *mbes,
                           bool onsync );
 static void bes_calc_regs( MatroxDriverData *mdrv, MatroxBesLayerData *mbes,
-                           CoreLayerRegionConfig *config, CoreSurface *surface );
+                           CoreLayerRegionConfig *config, CoreSurface *surface,
+                           bool front );
 
 #define BES_SUPPORTED_OPTIONS   (DLOP_DEINTERLACING | DLOP_DST_COLORKEY)
 
@@ -127,7 +128,6 @@ besInitLayer( CoreLayer                  *layer,
 
      /* set capabilities and type */
      description->caps = DLCAPS_SCREEN_LOCATION | DLCAPS_SURFACE |
-                         DLCAPS_BRIGHTNESS | DLCAPS_CONTRAST |
                          DLCAPS_DEINTERLACING | DLCAPS_DST_COLORKEY;
      description->type = DLTF_GRAPHICS | DLTF_VIDEO | DLTF_STILL_PICTURE;
 
@@ -145,11 +145,19 @@ besInitLayer( CoreLayer                  *layer,
      config->buffermode  = DLBM_FRONTONLY;
      config->options     = DLOP_NONE;
 
-     /* fill out default color adjustment,
-        only fields set in flags will be accepted from applications */
-     adjustment->flags      = DCAF_BRIGHTNESS | DCAF_CONTRAST;
-     adjustment->brightness = 0x8000;
-     adjustment->contrast   = 0x8000;
+     adjustment->flags   = DCAF_NONE;
+
+     if (mdrv->accelerator != FB_ACCEL_MATROX_MGAG200) {
+          description->caps      |= DLCAPS_BRIGHTNESS | DLCAPS_CONTRAST;
+
+          /* fill out default color adjustment,
+             only fields set in flags will be accepted from applications */
+          adjustment->flags      |= DCAF_BRIGHTNESS | DCAF_CONTRAST;
+          adjustment->brightness  = 0x8000;
+          adjustment->contrast    = 0x8000;
+
+          mga_out32( mmio, 0x80, BESLUMACTL );
+     }
 
      /* disable backend scaler */
      mga_out32( mmio, 0, BESCTL );
@@ -165,8 +173,6 @@ besInitLayer( CoreLayer                  *layer,
      mga_out_dac( mmio, XCOLKEY0GREEN, 0x00 );
      mga_out_dac( mmio, XCOLKEY0BLUE,  0x00 );
 
-     mga_out32( mmio, 0x80, BESLUMACTL );
-
      return DFB_OK;
 }
 
@@ -177,20 +183,28 @@ besTestRegion( CoreLayer                  *layer,
                CoreLayerRegionConfig      *config,
                CoreLayerRegionConfigFlags *failed )
 {
-     MatroxDriverData           *mdrv      = (MatroxDriverData*) driver_data;
-     MatroxDeviceData           *mdev      = mdrv->device_data;
-     int                         max_width = mdev->g450_matrox ? 2048 : 1024;
-     CoreLayerRegionConfigFlags  fail      = 0;
+     MatroxDriverData           *mdrv       = (MatroxDriverData*) driver_data;
+     MatroxDeviceData           *mdev       = mdrv->device_data;
+     int                         max_width  = mdev->g450_matrox ? 2048 : 1024;
+     int                         max_height = 1024;
+     CoreLayerRegionConfigFlags  fail       = 0;
 
      if (config->options & ~BES_SUPPORTED_OPTIONS)
           fail |= CLRCF_OPTIONS;
 
+     if (config->options & DLOP_DEINTERLACING) {
+          /* make sure BESPITCH < 4096 */
+          max_width = mdev->g450_matrox ? 2048 - 128 : 1024;
+          max_height = 2048;
+     }
+
      switch (config->format) {
           case DSPF_YUY2:
           case DSPF_NV12:
           case DSPF_NV21:
                break;
 
+          case DSPF_ARGB:
           case DSPF_RGB32:
                if (!mdev->g450_matrox)
                     max_width = 512;
@@ -224,7 +238,7 @@ besTestRegion( CoreLayer                  *layer,
      if (config->width > max_width || config->width < 1)
           fail |= CLRCF_WIDTH;
 
-     if (config->height > 1024 || config->height < 1)
+     if (config->height > max_height || config->height < 1)
           fail |= CLRCF_HEIGHT;
 
      if (failed)
@@ -257,7 +271,7 @@ besSetRegion( CoreLayer                  *layer,
      if (updated & (CLRCF_WIDTH | CLRCF_HEIGHT | CLRCF_FORMAT |
                     CLRCF_OPTIONS | CLRCF_DEST | CLRCF_OPACITY | CLRCF_SOURCE))
      {
-          bes_calc_regs( mdrv, mbes, config, surface );
+          bes_calc_regs( mdrv, mbes, config, surface, true );
           bes_set_regs( mdrv, mbes, true );
      }
 
@@ -315,11 +329,11 @@ besFlipRegion( CoreLayer           *layer,
      MatroxDriverData   *mdrv = (MatroxDriverData*) driver_data;
      MatroxBesLayerData *mbes = (MatroxBesLayerData*) layer_data;
 
-     dfb_surface_flip_buffers( surface, false );
-
-     bes_calc_regs( mdrv, mbes, &mbes->config, surface );
+     bes_calc_regs( mdrv, mbes, &mbes->config, surface, false );
      bes_set_regs( mdrv, mbes, flags & DSFLIP_ONSYNC );
 
+     dfb_surface_flip_buffers( surface, false );
+
      if (flags & DSFLIP_WAIT)
           dfb_screen_wait_vsync( mdrv->primary );
 
@@ -335,6 +349,9 @@ besSetColorAdjustment( CoreLayer          *layer,
      MatroxDriverData *mdrv = (MatroxDriverData*) driver_data;
      volatile __u8    *mmio = mdrv->mmio_base;
 
+     if (mdrv->accelerator == FB_ACCEL_MATROX_MGAG200)
+          return DFB_UNSUPPORTED;
+
      mga_out32( mmio, (adj->contrast >> 8) |
                       ((__u8)(((int)adj->brightness >> 8) - 128)) << 16,
                 BESLUMACTL );
@@ -381,20 +398,18 @@ static void bes_set_regs( MatroxDriverData *mdrv, MatroxBesLayerData *mbes,
 {
      int            line = 0;
      volatile __u8 *mmio = mdrv->mmio_base;
+     VideoMode     *mode = dfb_system_current_mode();
 
-     if (!onsync) {
-          VideoMode *current_mode = dfb_system_current_mode();
-
-          /* FIXME: I don't think this should be NULL ever. */
-          if (!current_mode)
+     if (!mode) {
+          mode = dfb_system_modes();
+          if (!mode)
                return;
-
-          line = mga_in32( mmio, MGAREG_VCOUNT ) + 48;
-
-          if (line > current_mode->yres)
-               line = current_mode->yres;
      }
 
+     line = onsync ? mode->yres : (mga_in32( mmio, MGAREG_VCOUNT ) + 48);
+     if (line > mode->yres)
+          line = mode->yres;
+
      mga_out32( mmio, mbes->regs.besGLOBCTL | (line << 16), BESGLOBCTL);
 
      mga_out32( mmio, mbes->regs.besA1ORG, BESA1ORG );
@@ -433,19 +448,21 @@ static void bes_set_regs( MatroxDriverData *mdrv, MatroxBesLayerData *mbes,
 static void bes_calc_regs( MatroxDriverData      *mdrv,
                            MatroxBesLayerData    *mbes,
                            CoreLayerRegionConfig *config,
-                           CoreSurface           *surface )
+                           CoreSurface           *surface,
+                           bool                   front )
 {
      MatroxDeviceData *mdev = mdrv->device_data;
      int cropleft, cropright, croptop, cropbot, croptop_uv;
      int pitch, tmp, hzoom, intrep, field_height;
      DFBRectangle   source, dest;
-     DFBRegion      dstBox;
-     SurfaceBuffer *front_buffer = surface->front_buffer;
-     VideoMode     *current_mode = dfb_system_current_mode();
+     DFBRegion      dst;
+     bool           visible;
+     SurfaceBuffer *buffer = front ? surface->front_buffer : surface->back_buffer;
+     VideoMode     *mode   = dfb_system_current_mode();
 
-     if (!current_mode) {
-          current_mode = dfb_system_modes();
-          if (!current_mode) {
+     if (!mode) {
+          mode = dfb_system_modes();
+          if (!mode) {
                D_BUG( "No current and no default mode" );
                return;
           }
@@ -454,10 +471,10 @@ static void bes_calc_regs( MatroxDriverData      *mdrv,
      source = config->source;
      dest   = config->dest;
 
-     if (!mdev->g450_matrox && surface->format == DSPF_RGB32)
+     if (!mdev->g450_matrox && (surface->format == DSPF_RGB32 || surface->format == DSPF_ARGB))
           dest.w = source.w;
 
-     pitch = front_buffer->video.pitch;
+     pitch = buffer->video.pitch;
 
      field_height = surface->height;
 
@@ -469,11 +486,19 @@ static void bes_calc_regs( MatroxDriverData      *mdrv,
      } else
           mbes->regs.besCTL_field = 0;
 
+     /* destination region */
+     dst.x1 = dest.x;
+     dst.y1 = dest.y;
+     dst.x2 = dest.x + dest.w - 1;
+     dst.y2 = dest.y + dest.h - 1;
+
+     visible = dfb_region_intersect( &dst, 0, 0, mode->xres - 1, mode->yres - 1 );
+
      /* calculate destination cropping */
      cropleft  = -dest.x;
      croptop   = -dest.y;
-     cropright = dest.x + dest.w - current_mode->xres;
-     cropbot   = dest.y + dest.h - current_mode->yres;
+     cropright = dest.x + dest.w - mode->xres;
+     cropbot   = dest.y + dest.h - mode->yres;
 
      cropleft   = cropleft > 0 ? cropleft : 0;
      croptop    = croptop > 0 ? croptop : 0;
@@ -481,12 +506,6 @@ static void bes_calc_regs( MatroxDriverData      *mdrv,
      cropbot    = cropbot > 0 ? cropbot : 0;
      croptop_uv = croptop;
 
-     /* destination box */
-     dstBox.x1 = dest.x + cropleft;
-     dstBox.y1 = dest.y + croptop;
-     dstBox.x2 = dest.x + dest.w - 1 - cropright;
-     dstBox.y2 = dest.y + dest.h - 1 - cropbot;
-
      /* scale crop values to source dimensions */
      if (cropleft)
           cropleft = ((__u64) (source.w << 16) * cropleft / dest.w) & ~0x3;
@@ -501,15 +520,15 @@ static void bes_calc_regs( MatroxDriverData      *mdrv,
 
      /* should horizontal zoom be used? */
      if (mdev->g450_matrox)
-          hzoom = (1000000/current_mode->pixclock >= 234) ? 1 : 0;
+          hzoom = (1000000/mode->pixclock >= 234) ? 1 : 0;
      else
-          hzoom = (1000000/current_mode->pixclock >= 135) ? 1 : 0;
+          hzoom = (1000000/mode->pixclock >= 135) ? 1 : 0;
 
      /* initialize */
      mbes->regs.besGLOBCTL = 0;
 
      /* enable/disable depending on opacity */
-     mbes->regs.besCTL = config->opacity ? 1 : 0;
+     mbes->regs.besCTL = (config->opacity && visible) ? BESEN : 0;
 
      /* pixel format settings */
      switch (surface->format) {
@@ -548,6 +567,7 @@ static void bes_calc_regs( MatroxDriverData      *mdrv,
                mbes->regs.besGLOBCTL |= BESRGB16;
                break;
 
+          case DSPF_ARGB:
           case DSPF_RGB32:
                mbes->regs.besGLOBCTL |= BESRGB32;
                break;
@@ -557,46 +577,49 @@ static void bes_calc_regs( MatroxDriverData      *mdrv,
                return;
      }
 
-     mbes->regs.besGLOBCTL |= 3*hzoom | (current_mode->yres & 0xFFF) << 16;
+     if (surface->width > 1024)
+          mbes->regs.besCTL &= ~BESVFEN;
+
+     mbes->regs.besGLOBCTL |= 3*hzoom | (mode->yres & 0xFFF) << 16;
 
      mbes->regs.besPITCH = pitch / DFB_BYTES_PER_PIXEL(surface->format);
 
      /* buffer offsets */
-     mbes->regs.besA1ORG = front_buffer->video.offset +
+     mbes->regs.besA1ORG = buffer->video.offset +
                            pitch * (source.y + (croptop >> 16));
      mbes->regs.besA2ORG = mbes->regs.besA1ORG +
-                           front_buffer->video.pitch;
+                           buffer->video.pitch;
 
      switch (surface->format) {
           case DSPF_NV12:
           case DSPF_NV21:
-               mbes->regs.besA1CORG  = front_buffer->video.offset +
-                                       surface->height * front_buffer->video.pitch +
+               mbes->regs.besA1CORG  = buffer->video.offset +
+                                       surface->height * buffer->video.pitch +
                                        pitch * (source.y/2 + (croptop_uv >> 16));
                mbes->regs.besA2CORG  = mbes->regs.besA1CORG +
-                                       front_buffer->video.pitch;
+                                       buffer->video.pitch;
                break;
 
           case DSPF_I420:
           case DSPF_YV12:
-               mbes->regs.besA1CORG  = front_buffer->video.offset +
-                                       surface->height * front_buffer->video.pitch +
+               mbes->regs.besA1CORG  = buffer->video.offset +
+                                       surface->height * buffer->video.pitch +
                                        pitch/2 * (source.y/2 + (croptop_uv >> 16));
                mbes->regs.besA2CORG  = mbes->regs.besA1CORG +
-                                       front_buffer->video.pitch/2;
+                                       buffer->video.pitch/2;
 
                mbes->regs.besA1C3ORG = mbes->regs.besA1CORG +
-                                       surface->height/2 * front_buffer->video.pitch/2;
+                                       surface->height/2 * buffer->video.pitch/2;
                mbes->regs.besA2C3ORG = mbes->regs.besA1C3ORG +
-                                       front_buffer->video.pitch/2;
+                                       buffer->video.pitch/2;
                break;
 
           default:
                ;
      }
 
-     mbes->regs.besHCOORD   = (dstBox.x1 << 16) | dstBox.x2;
-     mbes->regs.besVCOORD   = (dstBox.y1 << 16) | dstBox.y2;
+     mbes->regs.besHCOORD   = (dst.x1 << 16) | dst.x2;
+     mbes->regs.besVCOORD   = (dst.y1 << 16) | dst.y2;
 
      mbes->regs.besHSRCST   = (source.x << 16) + cropleft;
      mbes->regs.besHSRCEND  = ((source.x + source.w - 1) << 16) - cropright;
@@ -624,7 +647,7 @@ static void bes_calc_regs( MatroxDriverData      *mdrv,
           field_height - 1 - source.y - (croptop >> 16);
 
      /* horizontal scaling */
-     if (!mdev->g450_matrox && surface->format == DSPF_RGB32) {
+     if (!mdev->g450_matrox && (surface->format == DSPF_RGB32 || surface->format == DSPF_ARGB)) {
           mbes->regs.besHISCAL   = 0x20000 << hzoom;
           mbes->regs.besHSRCST  *= 2;
           mbes->regs.besHSRCEND *= 2;
