@@ -47,7 +47,7 @@
 
 #include "matrox_state.h"
 
-#define MGA_KEYMASK(format)   ((1 << MIN(24, DFB_BITS_PER_PIXEL(format))) - 1)
+#define MGA_KEYMASK(format)   ((1 << DFB_COLOR_BITS_PER_PIXEL(format)) - 1)
 
 void matrox_set_destination( MatroxDriverData *mdrv,
                              MatroxDeviceData *mdev,
@@ -59,7 +59,6 @@ void matrox_set_destination( MatroxDriverData *mdrv,
      int            bytes_per_pixel = DFB_BYTES_PER_PIXEL(buffer->format);
 
      mdev->dst_pitch = buffer->video.pitch / bytes_per_pixel;
-     mdev->dst_format = destination->format;
 
      D_ASSERT( mdev->dst_pitch % 32 == 0 );
 
@@ -67,7 +66,7 @@ void matrox_set_destination( MatroxDriverData *mdrv,
 
      mdev->depth_buffer = depth_buffer != NULL;
 
-     if (mdev->dst_format == DSPF_YUY2 || mdev->dst_format == DSPF_UYVY)
+     if (destination->format == DSPF_YUY2 || destination->format == DSPF_UYVY)
           mdev->dst_pitch /= 2;
 
      if (mdev->old_matrox) {
@@ -149,27 +148,31 @@ void matrox_set_clip( MatroxDriverData *mdrv,
           mga_out32( mmio, (mdev->dst_pitch * clip->y2) & 0xFFFFFF, YBOT );
      }
 
-     if (mdev->dst_format == DSPF_YUY2 || mdev->dst_format == DSPF_UYVY)
-          mga_out32( mmio, ((clip->x2/2 & 0x0FFF) << 16) | (clip->x1/2 & 0x0FFF), CXBNDRY );
-     else
-          mga_out32( mmio, ((clip->x2 & 0x0FFF) << 16) | (clip->x1 & 0x0FFF), CXBNDRY );
+     mga_out32( mmio, ((clip->x2 & 0x0FFF) << 16) | (clip->x1 & 0x0FFF), CXBNDRY );
 }
 
 void matrox_validate_drawColor( MatroxDriverData *mdrv,
                                 MatroxDeviceData *mdev,
                                 CardState        *state )
 {
-     volatile __u8 *mmio = mdrv->mmio_base;
+     DFBColor       color = state->color;
+     volatile __u8 *mmio  = mdrv->mmio_base;
 
      if (MGA_IS_VALID( m_drawColor ))
           return;
 
+     if (state->drawingflags & DSDRAW_SRC_PREMULTIPLY) {
+          color.r = (color.r * (color.a + 1)) >> 8;
+          color.g = (color.g * (color.a + 1)) >> 8;
+          color.b = (color.b * (color.a + 1)) >> 8;
+     }
+
      mga_waitfifo( mdrv, mdev, 4 );
 
-     mga_out32( mmio, U8_TO_F0915(state->color.a), ALPHASTART );
-     mga_out32( mmio, U8_TO_F0915(state->color.r), DR4 );
-     mga_out32( mmio, U8_TO_F0915(state->color.g), DR8 );
-     mga_out32( mmio, U8_TO_F0915(state->color.b), DR12 );
+     mga_out32( mmio, U8_TO_F0915(color.a), ALPHASTART );
+     mga_out32( mmio, U8_TO_F0915(color.r), DR4 );
+     mga_out32( mmio, U8_TO_F0915(color.g), DR8 );
+     mga_out32( mmio, U8_TO_F0915(color.b), DR12 );
 
      MGA_VALIDATE( m_drawColor );
      MGA_INVALIDATE( m_blitColor );
@@ -216,119 +219,126 @@ void matrox_validate_color( MatroxDriverData *mdrv,
                             MatroxDeviceData *mdev,
                             CardState        *state )
 {
-     volatile __u8 *mmio = mdrv->mmio_base;
+     DFBColor       color = state->color;
+     volatile __u8 *mmio  = mdrv->mmio_base;
 
-     __u32 color;
+     __u32 fcol;
      __u8  cb, cr;
 
      if (MGA_IS_VALID( m_color ))
           return;
 
+     if (state->drawingflags & DSDRAW_SRC_PREMULTIPLY) {
+          color.r = (color.r * (color.a + 1)) >> 8;
+          color.g = (color.g * (color.a + 1)) >> 8;
+          color.b = (color.b * (color.a + 1)) >> 8;
+     }
+
      switch (state->destination->format) {
           case DSPF_ALUT44:
-               color = (state->color.a & 0xF0) | state->color_index;
-               color |= color << 8;
-               color |= color << 16;
+               fcol = (color.a & 0xF0) | state->color_index;
+               fcol |= fcol << 8;
+               fcol |= fcol << 16;
                break;
           case DSPF_LUT8:
-               color = state->color_index;
-               color |= color << 8;
-               color |= color << 16;
+               fcol = state->color_index;
+               fcol |= fcol << 8;
+               fcol |= fcol << 16;
                break;
           case DSPF_RGB332:
-               color = PIXEL_RGB332( state->color.r,
-                                     state->color.g,
-                                     state->color.b );
-               color |= color << 8;
-               color |= color << 16;
+               fcol = PIXEL_RGB332( color.r,
+                                    color.g,
+                                    color.b );
+               fcol |= fcol << 8;
+               fcol |= fcol << 16;
                break;
           case DSPF_ARGB4444:
-               color = PIXEL_ARGB4444( state->color.a,
-                                       state->color.r,
-                                       state->color.g,
-                                       state->color.b );
-               color |= color << 16;
+               fcol = PIXEL_ARGB4444( color.a,
+                                      color.r,
+                                      color.g,
+                                      color.b );
+               fcol |= fcol << 16;
                break;
           case DSPF_ARGB1555:
-               color = PIXEL_ARGB1555( state->color.a,
-                                       state->color.r,
-                                       state->color.g,
-                                       state->color.b );
-               color |= color << 16;
+               fcol = PIXEL_ARGB1555( color.a,
+                                      color.r,
+                                      color.g,
+                                      color.b );
+               fcol |= fcol << 16;
                break;
           case DSPF_RGB16:
-               color = PIXEL_RGB16( state->color.r,
-                                    state->color.g,
-                                    state->color.b );
-               color |= color << 16;
+               fcol = PIXEL_RGB16( color.r,
+                                   color.g,
+                                   color.b );
+               fcol |= fcol << 16;
                break;
           case DSPF_RGB24:
-               color = PIXEL_RGB32( state->color.r,
-                                    state->color.g,
-                                    state->color.b );
-               color |= color << 24;
+               fcol = PIXEL_RGB32( color.r,
+                                   color.g,
+                                   color.b );
+               fcol |= fcol << 24;
                break;
           case DSPF_RGB32:
-               color = PIXEL_RGB32( state->color.r,
-                                    state->color.g,
-                                    state->color.b );
+               fcol = PIXEL_RGB32( color.r,
+                                   color.g,
+                                   color.b );
                break;
           case DSPF_ARGB:
-               color = PIXEL_ARGB( state->color.a,
-                                   state->color.r,
-                                   state->color.g,
-                                   state->color.b );
+               fcol = PIXEL_ARGB( color.a,
+                                  color.r,
+                                  color.g,
+                                  color.b );
                break;
           case DSPF_A8:
-               color = state->color.a;
-               color |= color << 8;
-               color |= color << 16;
+               fcol = color.a;
+               fcol |= fcol << 8;
+               fcol |= fcol << 16;
                break;
           case DSPF_I420:
           case DSPF_YV12:
-               RGB_TO_YCBCR( state->color.r,
-                             state->color.g,
-                             state->color.b,
-                             color, cb, cr );
-               color |= color << 8;
-               color |= color << 16;
-               mdev->color[0] = color;
+               RGB_TO_YCBCR( color.r,
+                             color.g,
+                             color.b,
+                             fcol, cb, cr );
+               fcol |= fcol << 8;
+               fcol |= fcol << 16;
+               mdev->color[0] = fcol;
                mdev->color[1] = (cb << 24) | (cb << 16) | (cb << 8) | cb;
                mdev->color[2] = (cr << 24) | (cr << 16) | (cr << 8) | cr;
                break;
           case DSPF_NV12:
-               RGB_TO_YCBCR( state->color.r,
-                             state->color.g,
-                             state->color.b,
-                             color, cb, cr );
-               color |= color << 8;
-               color |= color << 16;
-               mdev->color[0] = color;
+               RGB_TO_YCBCR( color.r,
+                             color.g,
+                             color.b,
+                             fcol, cb, cr );
+               fcol |= fcol << 8;
+               fcol |= fcol << 16;
+               mdev->color[0] = fcol;
                mdev->color[1] = (cr << 24) | (cb << 16) | (cr << 8) | cb;
                break;
           case DSPF_NV21:
-               RGB_TO_YCBCR( state->color.r,
-                             state->color.g,
-                             state->color.b,
-                             color, cb, cr );
-               color |= color << 8;
-               color |= color << 16;
-               mdev->color[0] = color;
+               RGB_TO_YCBCR( color.r,
+                             color.g,
+                             color.b,
+                             fcol, cb, cr );
+               fcol |= fcol << 8;
+               fcol |= fcol << 16;
+               mdev->color[0] = fcol;
                mdev->color[1] = (cb << 24) | (cr << 16) | (cb << 8) | cr;
                break;
           case DSPF_YUY2:
-               RGB_TO_YCBCR( state->color.r,
-                             state->color.g,
-                             state->color.b,
-                             color, cb, cr );
-               color = PIXEL_YUY2( color, cb, cr );
+               RGB_TO_YCBCR( color.r,
+                             color.g,
+                             color.b,
+                             fcol, cb, cr );
+               fcol = PIXEL_YUY2( fcol, cb, cr );
                break;
           case DSPF_UYVY:
-               RGB_TO_YCBCR( state->color.r,
-                             state->color.g,
-                             state->color.b,
-                             color, cb, cr );
-               color = PIXEL_UYVY( color, cb, cr );
+               RGB_TO_YCBCR( color.r,
+                             color.g,
+                             color.b,
+                             fcol, cb, cr );
+               fcol = PIXEL_UYVY( fcol, cb, cr );
                break;
           default:
                D_BUG( "unexpected pixelformat!" );
@@ -336,7 +346,7 @@ void matrox_validate_color( MatroxDriverData *mdrv,
      }
 
      mga_waitfifo( mdrv, mdev, 1 );
-     mga_out32( mmio, color, FCOL );
+     mga_out32( mmio, fcol, FCOL );
 
      MGA_VALIDATE( m_color );
      MGA_INVALIDATE( m_srckey );
@@ -413,9 +423,15 @@ void matrox_validate_blitBlend( MatroxDriverData *mdrv,
      if (state->blittingflags & (DSBLIT_BLEND_ALPHACHANNEL |
                                  DSBLIT_BLEND_COLORALPHA))
      {
-          alphactrl = matroxSourceBlend[state->src_blend - 1] |
-                      matroxDestBlend  [state->dst_blend - 1] |
-                      ALPHACHANNEL;
+          if (state->blittingflags & DSBLIT_SRC_PREMULTIPLY)
+               /* src_blend == ONE and dst_blend == INVSRCALPHA/INVSRCCOLOR */
+               alphactrl = matroxSourceBlend[DSBF_SRCALPHA    - 1] |
+                           matroxDestBlend  [state->dst_blend - 1] |
+                           VIDEOALPHA;
+          else
+               alphactrl = matroxSourceBlend[state->src_blend - 1] |
+                           matroxDestBlend  [state->dst_blend - 1] |
+                           ALPHACHANNEL;
 
           if (state->source->format == DSPF_RGB32) {
                alphactrl |= DIFFUSEDALPHA;
@@ -515,7 +531,7 @@ void matrox_validate_Source( MatroxDriverData *mdrv,
      mdev->w = surface->width;
      mdev->h = surface->height;
 
-     if (mdev->dst_format == DSPF_YUY2 || mdev->dst_format == DSPF_UYVY) {
+     if (state->destination->format == DSPF_YUY2 || state->destination->format == DSPF_UYVY) {
           mdev->w /= 2;
           mdev->src_pitch /= 2;
      }
@@ -555,10 +571,10 @@ void matrox_validate_Source( MatroxDriverData *mdrv,
 
      switch (surface->format) {
           case DSPF_YUY2:
-               texctl |= (mdev->dst_format == DSPF_YUY2) ? TW32 : TW422;
+               texctl |= (state->destination->format == DSPF_YUY2) ? TW32 : TW422;
                break;
           case DSPF_UYVY:
-               texctl |= (mdev->dst_format == DSPF_UYVY) ? TW32 : TW422UYVY;
+               texctl |= (state->destination->format == DSPF_UYVY) ? TW32 : TW422UYVY;
                break;
           case DSPF_I420:
           case DSPF_YV12:
@@ -650,7 +666,7 @@ void matrox_validate_source( MatroxDriverData *mdrv,
 
      D_ASSERT( buffer->video.offset % 64 == 0 );
 
-     if (mdev->dst_format == DSPF_YUY2 || mdev->dst_format == DSPF_UYVY)
+     if (state->destination->format == DSPF_YUY2 || state->destination->format == DSPF_UYVY)
           mdev->src_pitch /= 2;
 
      if (mdev->old_matrox) {
@@ -685,8 +701,13 @@ void matrox_validate_SrcKey( MatroxDriverData *mdrv,
      if (MGA_IS_VALID( m_SrcKey ))
           return;
 
-     mask = MGA_KEYMASK(surface->format);
-     key  = state->src_colorkey & mask;
+     if (state->blittingflags & DSBLIT_SRC_COLORKEY) {
+          mask = MGA_KEYMASK(surface->format);
+          key  = state->src_colorkey & mask;
+     } else {
+          mask = 0;
+          key  = 0xFFFF;
+     }
 
      mga_waitfifo( mdrv, mdev, 2);
 
@@ -711,31 +732,19 @@ void matrox_validate_srckey( MatroxDriverData *mdrv,
      mask = MGA_KEYMASK(surface->format);
      key  = state->src_colorkey & mask;
 
-     mga_waitfifo( mdrv, mdev, 2);
-
-     if (DFB_BYTES_PER_PIXEL(state->source->format) > 2)
-          mga_out32( mmio, mask, BCOL );
-     else
-          mga_out32( mmio, 0xFFFFFFFF, BCOL );
-
      switch (DFB_BYTES_PER_PIXEL(state->source->format)) {
           case 1:
-               mga_out32( mmio, key | (key <<  8) |
-                                      (key << 16) |
-                                      (key << 24), FCOL );
-               break;
+               mask |= mask << 8;
+               key  |= key  << 8;
           case 2:
-               mga_out32( mmio, key | (key << 16), FCOL );
-               break;
-          case 3:
-          case 4:
-               mga_out32( mmio, key, FCOL );
-               break;
-          default:
-               D_BUG( "unexpected bytes per pixel" );
-               break;
+               mask |= mask << 16;
+               key  |= key  << 16;
      }
 
+     mga_waitfifo( mdrv, mdev, 2);
+     mga_out32( mmio, mask, BCOL );
+     mga_out32( mmio, key, FCOL );
+
      MGA_VALIDATE( m_srckey );
      MGA_INVALIDATE( m_color );
 }
