@@ -35,7 +35,7 @@
 #include <fcntl.h>
 #include <sys/ioctl.h>
 
-#include <linux/fb.h>
+#include <fbdev/fb.h>
 
 #include <directfb.h>
 
@@ -48,8 +48,6 @@
 #include <core/gfxcard.h>
 #include <core/surfaces.h>
 
-#include <core/accelerators.h>
-
 #include <gfx/convert.h>
 #include <gfx/util.h>
 #include <misc/conf.h>
@@ -755,7 +753,8 @@ static DFBResult
 driver_init_driver( GraphicsDevice      *device,
                     GraphicsDeviceFuncs *funcs,
                     void                *driver_data,
-                    void                *device_data )
+                    void                *device_data,
+                    CoreDFB             *core )
 {
      TDFXDriverData *tdrv = (TDFXDriverData*) driver_data;
 
