@@ -2,11 +2,13 @@
 #define __UC_OVERLAY_H__
 
 #define UC_OVL_CAPS (DLCAPS_SURFACE | DLCAPS_OPACITY | DLCAPS_SCREEN_LOCATION \
-    | DLCAPS_DEINTERLACING | DLCAPS_DST_COLORKEY | DLCAPS_LEVELS)
+    | DLCAPS_DEINTERLACING | DLCAPS_DST_COLORKEY | DLCAPS_LEVELS \
+    | DLCAPS_FIELD_PARITY )
 /*    | DLCAPS_BRIGHTNESS | DLCAPS_CONTRAST              \
     | DLCAPS_SATURATION | DLCAPS_HUE)
     */
-#define UC_OVL_OPTIONS (DLOP_DEINTERLACING | DLOP_DST_COLORKEY)
+#define UC_OVL_OPTIONS (DLOP_DEINTERLACING | DLOP_DST_COLORKEY \
+    | DLOP_FIELD_PARITY | DLOP_OPACITY )
 
 #define ALIGN_TO(v, n) (((v) + (n-1)) & ~(n-1))
 #define UC_MAP_V1_FIFO_CONTROL(depth, pre_thr, thr) \
@@ -45,10 +47,15 @@ typedef struct _UcOverlayData {
 
     struct uc_ovl_vinfo v1;          // Video overlay V1
 
+    CoreLayerRegionConfig config;
+    
     bool                deinterlace;
     int                 field;
 
     CoreSurface        *surface;
+    
+    int                 opacity_primary; // overlay opacity if primary is logically
+                                         // above or -1 if primary has alpha channel
 
 } UcOverlayData;
 
@@ -57,7 +64,7 @@ typedef struct _UcOverlayData {
 
 bool uc_ovl_map_vzoom(int sh, int dh, __u32* zoom, __u32* mini);
 bool uc_ovl_map_hzoom(int sw, int dw,  __u32* zoom, __u32* mini,
-                      int* falign, int* dcount);
+                      __u32* falign, __u32* dcount);
 __u32 uc_ovl_map_qwpitch(int falign, DFBSurfacePixelFormat format, int sw);
 __u32 uc_ovl_map_format(DFBSurfacePixelFormat format);
 void uc_ovl_map_window(int scrw, int scrh, DFBRectangle* win, int sw, int sh,
