@@ -37,6 +37,13 @@
 #include <sys/ioctl.h>
 #include <sys/vt.h>
 
+#include <linux/version.h>
+
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)
+typedef unsigned long kernel_ulong_t;
+#define BITS_PER_LONG    (sizeof(long)*8)
+#endif
+
 #include <linux/joystick.h>
 
 #include <directfb.h>
