@@ -69,7 +69,7 @@ DIRECT_INTERFACE_IMPLEMENTATION( IDirectFBFont, FT2 )
 
 static FT_Library      library           = NULL;
 static int             library_ref_count = 0;
-static pthread_mutex_t library_mutex     = DIRECT_UTIL_RECURSIVE_PTHREAD_MUTEX_INITIALIZER;
+static pthread_mutex_t library_mutex     = PTHREAD_MUTEX_INITIALIZER;
 
 #define KERNING_CACHE_MIN   32
 #define KERNING_CACHE_MAX  127
@@ -508,7 +508,8 @@ Probe( IDirectFBFont_ProbeContext *ctx )
       * but due to freetype bugs it doesn't work.
       */
      err = FT_New_Face( library, ctx->filename, 0, &face );
-     FT_Done_Face( face );
+     if (!err)
+          FT_Done_Face( face );
      pthread_mutex_unlock ( &library_mutex );
 
      release_freetype();
