@@ -60,6 +60,18 @@ void direct_debug( const char *format, ... )  D_FORMAT_PRINTF(1);
 void direct_debug_at( DirectDebugDomain *domain,
                       const char        *format, ... )  D_FORMAT_PRINTF(2);
 
+void direct_debug_enter( DirectDebugDomain *domain,
+	      	         const char *func,
+                         const char *file,
+                         int         line,
+                         const char *format, ... )  D_FORMAT_PRINTF(5);
+
+void direct_debug_exit( DirectDebugDomain *domain,
+	      	        const char *func,
+                        const char *file,
+                        int         line,
+                        const char *format, ... )  D_FORMAT_PRINTF(5);
+
 void direct_break( const char *func,
                    const char *file,
                    int         line,
@@ -81,6 +93,11 @@ void direct_assumption( const char *exp,
 
 #if DIRECT_BUILD_TEXT && (DIRECT_BUILD_DEBUG || defined(DIRECT_FORCE_DEBUG))
 
+#if !DIRECT_BUILD_DEBUGS
+#warning Building with debug, but library headers suggest that debug is not supported.
+#endif
+
+
 #define D_DEBUG_ENABLED  (1)
 
 #ifdef HEAVYDEBUG
@@ -108,6 +125,16 @@ void direct_assumption( const char *exp,
           direct_debug_at( &d, x );                                                  \
      } while (0)
 
+#define D_DEBUG_ENTER(d,x...)                                                        \
+     do {                                                                            \
+          direct_debug_enter( &d, __FUNCTION__, __FILE__, __LINE__, x );             \
+     } while (0)
+
+#define D_DEBUG_EXIT(d,x...)                                                         \
+     do {                                                                            \
+          direct_debug_exit( &d, __FUNCTION__, __FILE__, __LINE__, x );              \
+     } while (0)
+
 #define D_ASSERT(exp)                                                                \
      do {                                                                            \
           if (!(exp))                                                                \
@@ -135,6 +162,8 @@ void direct_assumption( const char *exp,
 #define D_DEBUG_DOMAIN(i,n,d)
 #define D_DEBUG(x...)              do {} while (0)
 #define D_DEBUG_AT(d,x...)         do {} while (0)
+#define D_DEBUG_ENTER(d,x...)      do {} while (0)
+#define D_DEBUG_EXIT(d,x...)       do {} while (0)
 #define D_ASSERT(exp)              do {} while (0)
 #define D_ASSUME(exp)              do {} while (0)
 #define D_BREAK(x...)              do {} while (0)
@@ -159,6 +188,12 @@ void direct_assumption( const char *exp,
                                         (o)->magic = D_MAGIC(#m);                    \
                                    } while (0)
 
+#define D_MAGIC_SET_ONLY(o,m)      do {                                              \
+                                        D_ASSERT( (o) != NULL );                     \
+                                                                                     \
+                                        (o)->magic = D_MAGIC(#m);                    \
+                                   } while (0)
+
 #define D_MAGIC_ASSERT(o,m)        do {                                              \
                                         D_ASSERT( (o) != NULL );                     \
                                         D_ASSERT( (o)->magic == D_MAGIC(#m) );       \
