@@ -119,31 +119,8 @@ direct_util_align( int value,
 /*
  * Utility function to initialize recursive mutexes.
  */
-static inline int
-direct_util_recursive_pthread_mutex_init( pthread_mutex_t *mutex )
-{
-     int                 ret;
-     pthread_mutexattr_t attr;
+int direct_util_recursive_pthread_mutex_init( pthread_mutex_t *mutex );
 
-     pthread_mutexattr_init( &attr );
-#ifdef __USE_UNIX98
-     pthread_mutexattr_settype( &attr, PTHREAD_MUTEX_RECURSIVE );
-#endif
-     ret = pthread_mutex_init( mutex, &attr );
-     if (ret)
-          D_PERROR( "Fusion/Lock: Could not initialize recursive mutex!\n" );
-
-     pthread_mutexattr_destroy( &attr );
-
-     return ret;
-}
-
-#ifdef PTHREAD_RECURSIVE_MUTEX_INITIALIZER_NP
-#define DIRECT_UTIL_RECURSIVE_PTHREAD_MUTEX_INITIALIZER  PTHREAD_RECURSIVE_MUTEX_INITIALIZER_NP
-#else
-#warning PTHREAD_RECURSIVE_MUTEX_INITIALIZER_NP is not defined, be aware of dead locks
-#define DIRECT_UTIL_RECURSIVE_PTHREAD_MUTEX_INITIALIZER  PTHREAD_MUTEX_INITIALIZER
-#endif
 
 /* floor and ceil implementation to get rid of libm */
 
