@@ -1,13 +1,14 @@
 /*
    (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2002-2006  convergence GmbH.
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjl <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -69,9 +70,11 @@ DEFINE_MODULE_DIRECTORY( dfb_graphics_drivers, "gfxdrivers", DFB_GRAPHICS_DRIVER
  * struct for graphics cards
  */
 typedef struct {
-     /* amount of usable video memory */
+     /* amount of usable memory */
      unsigned int             videoram_length;
-
+     unsigned int             auxram_length;
+     unsigned int             auxram_offset;
+     
      char                    *module_name;
 
      GraphicsDriverInfo       driver_info;
@@ -87,7 +90,7 @@ typedef struct {
       * Points to the current state of the graphics card.
       */
      CardState               *state;
-     int                      holder; /* Fusion ID of state owner. */
+     FusionID                 holder; /* Fusion ID of state owner. */
 } GraphicsDeviceShared;
 
 struct _GraphicsDevice {
@@ -108,7 +111,7 @@ struct _GraphicsDevice {
 static GraphicsDevice *card = NULL;
 
 
-static void dfb_gfxcard_find_driver();
+static void dfb_gfxcard_find_driver( CoreDFB *core );
 static void dfb_gfxcard_load_driver();
 
 DFB_CORE_PART( gfxcard, sizeof(GraphicsDevice), sizeof(GraphicsDeviceShared) )
@@ -121,7 +124,9 @@ dfb_gfxcard_initialize( CoreDFB *core, void *data_local, void *data_shared )
 {
      DFBResult             ret;
      int                   videoram_length;
+     int                   auxram_length;
      GraphicsDeviceShared *shared;
+     FusionSHMPoolShared  *pool = dfb_core_shmpool( core );
 
      D_ASSERT( card == NULL );
 
@@ -146,12 +151,22 @@ dfb_gfxcard_initialize( CoreDFB *core, void *data_local, void *data_shared )
                shared->videoram_length = videoram_length;
      }
 
+     /* Limit auxiliary memory length (currently only AGP) */
+     auxram_length = dfb_system_auxram_length();
+     if (auxram_length) {
+          if (dfb_config->agpmem_limit > 0 &&
+              dfb_config->agpmem_limit < auxram_length)
+               shared->auxram_length = dfb_config->agpmem_limit;
+          else
+               shared->auxram_length = auxram_length;
+     }
+
      /* Build a list of available drivers. */
      direct_modules_explore_directory( &dfb_graphics_drivers );
 
      /* Load driver */
      if (dfb_system_caps() & CSCAPS_ACCELERATION)
-          dfb_gfxcard_find_driver();
+          dfb_gfxcard_find_driver( core );
 
      if (card->driver_funcs) {
           const GraphicsDriverFuncs *funcs = card->driver_funcs;
@@ -159,13 +174,13 @@ dfb_gfxcard_initialize( CoreDFB *core, void *data_local, void *data_shared )
           card->driver_data = D_CALLOC( 1, shared->driver_info.driver_data_size );
 
           card->device_data   =
-          shared->device_data = SHCALLOC( 1, shared->driver_info.device_data_size );
+          shared->device_data = SHCALLOC( pool, 1, shared->driver_info.device_data_size );
 
           ret = funcs->InitDriver( card, &card->funcs,
-                                   card->driver_data, card->device_data );
+                                   card->driver_data, card->device_data, core );
           if (ret) {
-               SHFREE( shared->device_data );
-               SHFREE( shared->module_name );
+               SHFREE( pool, shared->device_data );
+               SHFREE( pool, shared->module_name );
                D_FREE( card->driver_data );
                card = NULL;
                return ret;
@@ -175,8 +190,8 @@ dfb_gfxcard_initialize( CoreDFB *core, void *data_local, void *data_shared )
                                    card->driver_data, card->device_data );
           if (ret) {
                funcs->CloseDriver( card, card->driver_data );
-               SHFREE( shared->device_data );
-               SHFREE( shared->module_name );
+               SHFREE( pool, shared->device_data );
+               SHFREE( pool, shared->module_name );
                D_FREE( card->driver_data );
                card = NULL;
                return ret;
@@ -201,10 +216,20 @@ dfb_gfxcard_initialize( CoreDFB *core, void *data_local, void *data_shared )
      else
           card->caps = shared->device_info.caps;
 
-     shared->surface_manager = dfb_surfacemanager_create( shared->videoram_length,
+     shared->surface_manager = dfb_surfacemanager_create( core,
                                                           &shared->device_info.limits );
 
-     fusion_property_init( &shared->lock );
+     ret = dfb_surfacemanager_add_heap( shared->surface_manager, 
+                                        CSS_VIDEO, 0, shared->videoram_length );
+     /* FIXME: what to do in case of failure? */
+     
+     if (shared->auxram_length && card->caps.flags & CCF_AUXMEMORY) {
+          ret = dfb_surfacemanager_add_heap( shared->surface_manager, CSS_AUXILIARY,
+                                             shared->auxram_offset, shared->auxram_length );
+          /* FIXME: what to do in case of failure? */
+     }
+
+     fusion_property_init( &shared->lock, dfb_core_world(core) );
 
      return DFB_OK;
 }
@@ -241,7 +266,7 @@ dfb_gfxcard_join( CoreDFB *core, void *data_local, void *data_shared )
           card->device_data = shared->device_data;
 
           ret = funcs->InitDriver( card, &card->funcs,
-                                   card->driver_data, card->device_data );
+                                   card->driver_data, card->device_data, core );
           if (ret) {
                D_FREE( card->driver_data );
                card = NULL;
@@ -277,6 +302,7 @@ static DFBResult
 dfb_gfxcard_shutdown( CoreDFB *core, bool emergency )
 {
      GraphicsDeviceShared *shared;
+     FusionSHMPoolShared  *pool = dfb_core_shmpool( core );
 
      D_ASSERT( card != NULL );
      D_ASSERT( card->shared != NULL );
@@ -295,7 +321,7 @@ dfb_gfxcard_shutdown( CoreDFB *core, bool emergency )
 
           direct_module_unref( card->module );
 
-          SHFREE( card->device_data );
+          SHFREE( pool, card->device_data );
           D_FREE( card->driver_data );
      }
 
@@ -304,7 +330,7 @@ dfb_gfxcard_shutdown( CoreDFB *core, bool emergency )
      fusion_property_destroy( &shared->lock );
 
      if (shared->module_name)
-          SHFREE( shared->module_name );
+          SHFREE( pool, shared->module_name );
 
      card = NULL;
 
@@ -410,7 +436,7 @@ bool
 dfb_gfxcard_state_check( CardState *state, DFBAccelerationMask accel )
 {
      D_ASSERT( card != NULL );
-     D_ASSERT( state != NULL );
+     D_MAGIC_ASSERT( state, CardState );
 
      /*
       * If there's no CheckState function there's no acceleration at all.
@@ -507,7 +533,7 @@ dfb_gfxcard_state_check( CardState *state, DFBAccelerationMask accel )
      }
 
      /* Return whether the function bit is set. */
-     return (state->accel & accel);
+     return !!(state->accel & accel);
 }
 
 /*
@@ -519,11 +545,10 @@ dfb_gfxcard_state_acquire( CardState *state, DFBAccelerationMask accel )
 {
      GraphicsDeviceShared *shared;
      DFBSurfaceLockFlags   lock_flags;
-     int                   fid = fusion_id();
 
      D_ASSERT( card != NULL );
      D_ASSERT( card->shared != NULL );
-     D_ASSERT( state != NULL );
+     D_MAGIC_ASSERT( state, CardState );
 
      /* Destination may have been destroyed. */
      if (!state->destination)
@@ -593,13 +618,13 @@ dfb_gfxcard_state_acquire( CardState *state, DFBAccelerationMask accel )
      }
 
      /* if we are switching to another state... */
-     if (state != shared->state || fid != shared->holder) {
+     if (state != shared->state || state->fusion_id != shared->holder) {
           /* ...set all modification bits and clear 'set functions' */
           state->modified |= SMF_ALL;
           state->set       = 0;
 
           shared->state  = state;
-          shared->holder = fid;
+          shared->holder = state->fusion_id;
      }
 
      dfb_state_update( state, state->flags & CSF_SOURCE_LOCKED );
@@ -623,7 +648,7 @@ dfb_gfxcard_state_release( CardState *state )
 {
      D_ASSERT( card != NULL );
      D_ASSERT( card->shared != NULL );
-     D_ASSERT( state != NULL );
+     D_MAGIC_ASSERT( state, CardState );
      D_ASSERT( state->destination != NULL );
      D_ASSERT( state->destination->back_buffer != NULL );
 
@@ -659,7 +684,7 @@ dfb_gfxcard_fillrectangles( const DFBRectangle *rects, int num, CardState *state
 {
      D_ASSERT( card != NULL );
      D_ASSERT( card->shared != NULL );
-     D_ASSERT( state != NULL );
+     D_MAGIC_ASSERT( state, CardState );
      D_ASSERT( rects != NULL );
      D_ASSERT( num > 0 );
 
@@ -791,7 +816,7 @@ void dfb_gfxcard_drawrectangle( DFBRectangle *rect, CardState *state )
 
      D_ASSERT( card != NULL );
      D_ASSERT( card->shared != NULL );
-     D_ASSERT( state != NULL );
+     D_MAGIC_ASSERT( state, CardState );
      D_ASSERT( rect != NULL );
 
      dfb_state_lock( state );
@@ -848,7 +873,7 @@ void dfb_gfxcard_drawlines( DFBRegion *lines, int num_lines, CardState *state )
 
      D_ASSERT( card != NULL );
      D_ASSERT( card->shared != NULL );
-     D_ASSERT( state != NULL );
+     D_MAGIC_ASSERT( state, CardState );
      D_ASSERT( lines != NULL );
      D_ASSERT( num_lines > 0 );
 
@@ -890,7 +915,7 @@ void dfb_gfxcard_fillspans( int y, DFBSpan *spans, int num_spans, CardState *sta
 
      D_ASSERT( card != NULL );
      D_ASSERT( card->shared != NULL );
-     D_ASSERT( state != NULL );
+     D_MAGIC_ASSERT( state, CardState );
      D_ASSERT( spans != NULL );
      D_ASSERT( num_spans > 0 );
 
@@ -948,6 +973,12 @@ typedef struct {
                     dda.mi--;              \
                }                           \
           }                                \
+          else {                           \
+               dda.mi = 0;                 \
+               dda.mf = 0;                 \
+               dda.xf = 0;                 \
+               dda._2dy = 0;               \
+          }                                \
      } while (0)
 
 
@@ -973,6 +1004,8 @@ fill_tri( DFBTriangle *tri, CardState *state, bool accelerated )
      int clip_x1 = state->clip.x1;
      int clip_x2 = state->clip.x2;
 
+     D_MAGIC_ASSERT( state, CardState );
+
      y = tri->y1;
      yend = tri->y3;
 
@@ -1005,7 +1038,7 @@ fill_tri( DFBTriangle *tri, CardState *state, bool accelerated )
                rect.y = y;
                rect.h = 1;
 
-               if (rect.y >=  state->clip.y1) {
+               if (rect.w > 0 && rect.y >= state->clip.y1) {
                     if (accelerated)
                          card->funcs.FillRectangle( card->driver_data,
                                                     card->device_data, &rect );
@@ -1026,7 +1059,7 @@ void dfb_gfxcard_filltriangle( DFBTriangle *tri, CardState *state )
 {
      D_ASSERT( card != NULL );
      D_ASSERT( card->shared != NULL );
-     D_ASSERT( state != NULL );
+     D_MAGIC_ASSERT( state, CardState );
      D_ASSERT( tri != NULL );
 
      dfb_state_lock( state );
@@ -1074,7 +1107,7 @@ void dfb_gfxcard_blit( DFBRectangle *rect, int dx, int dy, CardState *state )
 
      D_ASSERT( card != NULL );
      D_ASSERT( card->shared != NULL );
-     D_ASSERT( state != NULL );
+     D_MAGIC_ASSERT( state, CardState );
      D_ASSERT( rect != NULL );
 
      dfb_state_lock( state );
@@ -1115,7 +1148,7 @@ void dfb_gfxcard_batchblit( DFBRectangle *rects, DFBPoint *points,
 
      D_ASSERT( card != NULL );
      D_ASSERT( card->shared != NULL );
-     D_ASSERT( state != NULL );
+     D_MAGIC_ASSERT( state, CardState );
      D_ASSERT( rects != NULL );
      D_ASSERT( points != NULL );
      D_ASSERT( num > 0 );
@@ -1177,7 +1210,7 @@ void dfb_gfxcard_tileblit( DFBRectangle *rect, int dx1, int dy1, int dx2, int dy
 
      D_ASSERT( card != NULL );
      D_ASSERT( card->shared != NULL );
-     D_ASSERT( state != NULL );
+     D_MAGIC_ASSERT( state, CardState );
      D_ASSERT( rect != NULL );
 
      /* If called with an invalid rectangle, the algorithm goes into an
@@ -1277,7 +1310,7 @@ void dfb_gfxcard_stretchblit( DFBRectangle *srect, DFBRectangle *drect,
 
      D_ASSERT( card != NULL );
      D_ASSERT( card->shared != NULL );
-     D_ASSERT( state != NULL );
+     D_MAGIC_ASSERT( state, CardState );
      D_ASSERT( srect != NULL );
      D_ASSERT( drect != NULL );
 
@@ -1323,7 +1356,7 @@ void dfb_gfxcard_texture_triangles( DFBVertex *vertices, int num,
      D_ASSERT( card->shared != NULL );
      D_ASSERT( vertices != NULL );
      D_ASSERT( num >= 3 );
-     D_ASSERT( state != NULL );
+     D_MAGIC_ASSERT( state, CardState );
 
      dfb_state_lock( state );
 
@@ -1354,6 +1387,8 @@ setup_font_state( CoreFont *font, CardState *state )
 {
      DFBSurfaceBlittingFlags flags = font->state.blittingflags;
 
+     D_MAGIC_ASSERT( state, CardState );
+
      /* set destination */
      dfb_state_set_destination( &font->state, state->destination );
 
@@ -1424,7 +1459,7 @@ dfb_gfxcard_drawstring( const __u8 *text, int bytes,
 
      D_ASSERT( card != NULL );
      D_ASSERT( card->shared != NULL );
-     D_ASSERT( state != NULL );
+     D_MAGIC_ASSERT( state, CardState );
      D_ASSERT( text != NULL );
      D_ASSERT( bytes > 0 );
      D_ASSERT( font != NULL );
@@ -1559,7 +1594,7 @@ void dfb_gfxcard_drawglyph( unichar index, int x, int y,
 
      D_ASSERT( card != NULL );
      D_ASSERT( card->shared != NULL );
-     D_ASSERT( state != NULL );
+     D_MAGIC_ASSERT( state, CardState );
      D_ASSERT( font != NULL );
 
      dfb_state_lock( state );
@@ -1618,7 +1653,7 @@ void dfb_gfxcard_drawstring_check_state( CoreFont *font, CardState *state )
 
      D_ASSERT( card != NULL );
      D_ASSERT( card->shared != NULL );
-     D_ASSERT( state != NULL );
+     D_MAGIC_ASSERT( state, CardState );
      D_ASSERT( font != NULL );
 
      dfb_state_lock( state );
@@ -1683,6 +1718,14 @@ void dfb_gfxcard_flush_texture_cache()
           card->funcs.FlushTextureCache( card->driver_data, card->device_data );
 }
 
+void dfb_gfxcard_flush_read_cache()
+{
+     D_ASSUME( card != NULL );
+
+     if (card && card->funcs.FlushReadCache)
+          card->funcs.FlushReadCache( card->driver_data, card->device_data );
+}
+
 void dfb_gfxcard_after_set_var()
 {
      D_ASSUME( card != NULL );
@@ -1691,6 +1734,22 @@ void dfb_gfxcard_after_set_var()
           card->funcs.AfterSetVar( card->driver_data, card->device_data );
 }
 
+void dfb_gfxcard_surface_enter( SurfaceBuffer *buffer, DFBSurfaceLockFlags flags )
+{
+     D_ASSUME( card != NULL );
+
+     if (card && card->funcs.SurfaceEnter)
+          card->funcs.SurfaceEnter( card->driver_data, card->device_data, buffer, flags );
+}
+
+void dfb_gfxcard_surface_leave( SurfaceBuffer *buffer )
+{
+     D_ASSUME( card != NULL );
+
+     if (card && card->funcs.SurfaceLeave)
+          card->funcs.SurfaceLeave( card->driver_data, card->device_data, buffer );
+}
+
 DFBResult
 dfb_gfxcard_adjust_heap_offset( int offset )
 {
@@ -1763,6 +1822,33 @@ dfb_gfxcard_reserve_memory( GraphicsDevice *device, unsigned int size )
      return shared->videoram_length;
 }
 
+int
+dfb_gfxcard_reserve_auxmemory( GraphicsDevice *device, unsigned int size )
+{
+     GraphicsDeviceShared *shared;
+     int                   offset;
+
+     D_ASSERT( device != NULL );
+     D_ASSERT( device->shared != NULL );
+
+     shared = device->shared;
+
+     if (shared->surface_manager)
+          return -1;
+
+     /* Reserve memory at the beginning of the aperture
+      * to prevent overflows on DMA buffers. */     
+
+     offset = shared->auxram_offset;
+
+     if (shared->auxram_length < (offset + size))
+          return -1;
+
+     shared->auxram_offset += size;
+
+     return offset;
+}
+
 unsigned int
 dfb_gfxcard_memory_length()
 {
@@ -1772,6 +1858,15 @@ dfb_gfxcard_memory_length()
      return card->shared->videoram_length;
 }
 
+unsigned int
+dfb_gfxcard_auxmemory_length()
+{
+     D_ASSERT( card != NULL );
+     D_ASSERT( card->shared != NULL );
+
+     return card->shared->auxram_length;
+}
+
 volatile void *
 dfb_gfxcard_map_mmio( GraphicsDevice *device,
                       unsigned int    offset,
@@ -1808,15 +1903,30 @@ dfb_gfxcard_memory_virtual( GraphicsDevice *device,
      return dfb_system_video_memory_virtual( offset );
 }
 
+unsigned long
+dfb_gfxcard_auxmemory_physical( GraphicsDevice *device,
+                                unsigned int    offset )
+{
+     return dfb_system_aux_memory_physical( offset );
+}
+
+void *
+dfb_gfxcard_auxmemory_virtual( GraphicsDevice *device,
+                               unsigned int    offset )
+{
+     return dfb_system_aux_memory_virtual( offset );
+}
+
 /** internal **/
 
 /*
  * loads/probes/unloads one driver module after another until a suitable
  * driver is found and returns its symlinked functions
  */
-static void dfb_gfxcard_find_driver()
+static void dfb_gfxcard_find_driver( CoreDFB *core )
 {
-     DirectLink *link;
+     DirectLink          *link;
+     FusionSHMPoolShared *pool = dfb_core_shmpool( core );
 
      direct_list_foreach (link, dfb_graphics_drivers.entries) {
           DirectModuleEntry *module = (DirectModuleEntry*) link;
@@ -1832,7 +1942,7 @@ static void dfb_gfxcard_find_driver()
                card->module       = module;
                card->driver_funcs = funcs;
 
-               card->shared->module_name = SHSTRDUP( module->name );
+               card->shared->module_name = SHSTRDUP( pool, module->name );
           }
           else
                direct_module_unref( module );
