@@ -40,7 +40,9 @@
 typedef enum {
      CCF_CLIPPING   = 0x00000001,
      CCF_NOTRIEMU   = 0x00000002,
-     CCF_READSYSMEM = 0x00000004
+     CCF_READSYSMEM = 0x00000004,
+     /* CCF_WRITESYSMEM ?! */
+     CCF_AUXMEMORY  = 0x00000010
 } CardCapabilitiesFlags;
 
 struct __DFB_CoreGraphicsSerial {
@@ -71,7 +73,7 @@ DECLARE_MODULE_DIRECTORY( dfb_graphics_drivers );
 /*
  * Increase this number when changes result in binary incompatibility!
  */
-#define DFB_GRAPHICS_DRIVER_ABI_VERSION          30
+#define DFB_GRAPHICS_DRIVER_ABI_VERSION          32
 
 #define DFB_GRAPHICS_DRIVER_INFO_NAME_LENGTH     40
 #define DFB_GRAPHICS_DRIVER_INFO_VENDOR_LENGTH   60
@@ -158,6 +160,23 @@ typedef struct _GraphicsDeviceFuncs {
       */
      void (*FlushTextureCache)( void *driver_data, void *device_data );
 
+     /*
+      * After the video memory has been written to by the accelerator
+      * make sure the CPU won't read back cached data.
+      */
+     void (*FlushReadCache)( void *driver_data, void *device_data );
+
+     /*
+      * Called before a software access to a video surface buffer.
+      */
+     void (*SurfaceEnter)( void *driver_data, void *device_data, 
+                           SurfaceBuffer *buffer, DFBSurfaceLockFlags flags );
+
+     /*
+      * Called after a software access to a video surface buffer.
+      */
+     void (*SurfaceLeave)( void *driver_data, void *device_data, SurfaceBuffer *buffer );
+
      /*
       * Return the serial of the last (queued) operation.
       *
@@ -232,7 +251,8 @@ typedef struct {
      DFBResult (*InitDriver)     (GraphicsDevice      *device,
                                   GraphicsDeviceFuncs *funcs,
                                   void                *driver_data,
-                                  void                *device_data);
+                                  void                *device_data,
+                                  CoreDFB             *core);
 
      DFBResult (*InitDevice)     (GraphicsDevice      *device,
                                   GraphicsDeviceInfo  *device_info,
@@ -302,19 +322,25 @@ void dfb_gfxcard_sync();
 void dfb_gfxcard_invalidate_state();
 void dfb_gfxcard_wait_serial( const CoreGraphicsSerial *serial );
 void dfb_gfxcard_flush_texture_cache();
+void dfb_gfxcard_flush_read_cache();
 void dfb_gfxcard_after_set_var();
+void dfb_gfxcard_surface_enter( SurfaceBuffer *buffer, DFBSurfaceLockFlags flags );
+void dfb_gfxcard_surface_leave( SurfaceBuffer *buffer );
 
 DFBResult dfb_gfxcard_adjust_heap_offset( int offset );
 
-SurfaceManager *dfb_gfxcard_surface_manager ();
-void            dfb_gfxcard_get_capabilities( CardCapabilities   *ret_caps );
-void            dfb_gfxcard_get_device_info ( GraphicsDeviceInfo *ret_info );
-void            dfb_gfxcard_get_driver_info ( GraphicsDriverInfo *ret_info );
+SurfaceManager *dfb_gfxcard_surface_manager   ();
+void            dfb_gfxcard_get_capabilities  ( CardCapabilities   *ret_caps );
+void            dfb_gfxcard_get_device_info   ( GraphicsDeviceInfo *ret_info );
+void            dfb_gfxcard_get_driver_info   ( GraphicsDriverInfo *ret_info );
 
-int             dfb_gfxcard_reserve_memory  ( GraphicsDevice      *device,
-                                              unsigned int         size );
+int             dfb_gfxcard_reserve_memory    ( GraphicsDevice      *device,
+                                                unsigned int         size );
+int             dfb_gfxcard_reserve_auxmemory ( GraphicsDevice      *device,
+                                                unsigned int         size );
 
-unsigned int    dfb_gfxcard_memory_length   ();
+unsigned int    dfb_gfxcard_memory_length     ();
+unsigned int    dfb_gfxcard_auxmemory_length  ();
 
 /*
  * Graphics drivers call this function to get access to MMIO regions.
@@ -341,8 +367,15 @@ void dfb_gfxcard_unmap_mmio( GraphicsDevice *device,
 
 int dfb_gfxcard_get_accelerator( GraphicsDevice *device );
 
-unsigned long  dfb_gfxcard_memory_physical( GraphicsDevice *device, unsigned int offset );
-void          *dfb_gfxcard_memory_virtual( GraphicsDevice *device, unsigned int offset );
+unsigned long  dfb_gfxcard_memory_physical( GraphicsDevice *device,
+                                            unsigned int    offset );
+void          *dfb_gfxcard_memory_virtual ( GraphicsDevice *device, 
+                                            unsigned int    offset );
+
+unsigned long  dfb_gfxcard_auxmemory_physical( GraphicsDevice *device,
+                                               unsigned int    offset );
+void          *dfb_gfxcard_auxmemory_virtual ( GraphicsDevice *device, 
+                                               unsigned int    offset );
 
 #endif
 
