@@ -46,6 +46,8 @@
 #include <core/layers_internal.h>
 #include <core/windows_internal.h>
 
+#include <fusion/shmalloc.h>
+
 #include <direct/messages.h>
 
 #include <direct/util.h>
@@ -118,12 +120,12 @@ context_destructor( FusionObject *object, bool zombie )
 /******************************************************************************/
 
 FusionObjectPool *
-dfb_layer_context_pool_create()
+dfb_layer_context_pool_create( const FusionWorld *world )
 {
      return fusion_object_pool_create( "Layer Context Pool",
                                        sizeof(CoreLayerContext),
                                        sizeof(CoreLayerContextNotification),
-                                       context_destructor );
+                                       context_destructor, world );
 }
 
 /******************************************************************************/
@@ -148,14 +150,16 @@ dfb_layer_context_create( CoreLayer         *layer,
 
      D_DEBUG_AT( Core_Layers, "%s -> %p\n", __FUNCTION__, context );
 
+     context->shmpool = shared->shmpool;
+
      /* Initialize the lock. */
-     if (fusion_skirmish_init( &context->lock, "Layer Context" )) {
+     if (fusion_skirmish_init( &context->lock, "Layer Context", dfb_core_world(layer->core) )) {
           fusion_object_destroy( &context->object );
           return DFB_FUSION;
      }
 
      /* Initialize the region vector. */
-     fusion_vector_init( &context->regions, 4 );
+     fusion_vector_init( &context->regions, 4, context->shmpool );
 
      /* Store layer ID, default configuration and default color adjustment. */
      context->layer_id   = shared->layer_id;
@@ -187,7 +191,7 @@ dfb_layer_context_create( CoreLayer         *layer,
      context->stack = dfb_windowstack_create( context );
      if (!context->stack) {
           dfb_layer_context_unref( context );
-          return DFB_NOSYSTEMMEMORY;
+          return D_OOSHM();
      }
 
      /* Tell the window stack about its size. */
