@@ -1,13 +1,14 @@
 /*
    (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2002-2006  convergence GmbH.
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjl <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -58,83 +59,99 @@
 
 D_DEBUG_DOMAIN( Core_SM, "Core/SurfaceMgr", "DirectFB Surface Manager" );
 
+typedef struct _Heap Heap;
+
 /*
  * initially there is one big free chunk,
  * chunks are splitted into a free and an occupied chunk if memory is allocated,
  * two chunks are merged to one free chunk if memory is deallocated
  */
 struct _Chunk {
-     int             magic;
+     int                  magic;
 
-     int             offset;      /* offset in video memory,
-                                     is greater or equal to the heap offset */
-     int             length;      /* length of this chunk in bytes */
+     int                  offset;      /* offset in memory,
+                                          is greater or equal to the heap offset */
+     int                  length;      /* length of this chunk in bytes */
+     
+     SurfaceBuffer       *buffer;      /* pointer to surface buffer occupying
+                                          this chunk, or NULL if chunk is free */
 
-     SurfaceBuffer  *buffer;      /* pointer to surface buffer occupying
-                                     this chunk, or NULL if chunk is free */
+     int                  tolerations; /* number of times this chunk was scanned
+                                          occupied, resetted in assure_video */
 
-     int             tolerations; /* number of times this chunk was scanned
-                                     occupied, resetted in assure_video */
+     Heap                *heap;        /* pointer to heap this chunk belongs to */
 
-     Chunk          *prev;
-     Chunk          *next;
+     Chunk               *prev;
+     Chunk               *next;
 };
 
+struct _Heap {
+     int                  magic;
+     
+     Chunk               *chunks;
+
+     CoreSurfaceStorage   storage;        /* storage of the heap 
+                                             (VIDEO or AUXILIARY) */
+     
+     int                  offset;
+     int                  length;         /* length of the heap in bytes */
+     int                  avail;          /* amount of available memory in bytes */
+
+     int                  min_toleration;
+     
+     Heap                *prev;
+     Heap                *next;
+} SurfaceHeap;
+
 struct _SurfaceManager {
-     int             magic;
+     int                  magic;
 
-     FusionSkirmish  lock;
+     FusionSkirmish       lock;
 
-     Chunk          *chunks;
-     int             length;
-     int             available;
+     FusionSHMPoolShared *shmpool;
 
-     int             min_toleration;
-
-     bool            suspended;
-
-     /* offset of the surface heap */
-     unsigned int    heap_offset;
+     Heap                *heaps;
+     
+     bool                 suspended;
 
      /* card limitations for surface offsets and their pitch */
-     unsigned int    byteoffset_align;
-     unsigned int    pixelpitch_align;
-     unsigned int    bytepitch_align;
+     unsigned int         byteoffset_align;
+     unsigned int         pixelpitch_align;
+     unsigned int         bytepitch_align;
 
-     unsigned int    max_power_of_two_pixelpitch;
-     unsigned int    max_power_of_two_bytepitch;
-     unsigned int    max_power_of_two_height;
+     unsigned int         max_power_of_two_pixelpitch;
+     unsigned int         max_power_of_two_bytepitch;
+     unsigned int         max_power_of_two_height;
 };
 
 
-static Chunk* split_chunk( Chunk *c, int length );
-static Chunk* free_chunk( SurfaceManager *manager, Chunk *chunk );
-static void occupy_chunk( SurfaceManager *manager, Chunk *chunk, SurfaceBuffer *buffer, int length );
+static Chunk *split_chunk ( SurfaceManager *manager,
+                            Chunk          *chunk,
+                            int             length );
+
+static Chunk *free_chunk  ( SurfaceManager *manager,
+                            Chunk          *chunk );
+
+static void   occupy_chunk( SurfaceManager *manager,
+                            Chunk          *chunk,
+                            SurfaceBuffer  *buffer,
+                            int             length );
 
 
 SurfaceManager *
-dfb_surfacemanager_create( unsigned int     length,
+dfb_surfacemanager_create( CoreDFB         *core,
                            CardLimitations *limits )
 {
-     Chunk          *chunk;
-     SurfaceManager *manager;
+     SurfaceManager      *manager;
+     FusionSHMPoolShared *pool;
 
-     manager = SHCALLOC( 1, sizeof(SurfaceManager) );
+     pool = dfb_core_shmpool( core );
+
+     manager = SHCALLOC( pool, 1, sizeof(SurfaceManager) );
      if (!manager)
           return NULL;
 
-     chunk = SHCALLOC( 1, sizeof(Chunk) );
-     if (!chunk) {
-          SHFREE( manager );
-          return NULL;
-     }
-
-     chunk->offset = 0;
-     chunk->length = length;
-
-     manager->chunks           = chunk;
-     manager->length           = length;
-     manager->available        = length;
+     manager->shmpool          = pool;
      manager->byteoffset_align = limits->surface_byteoffset_alignment;
      manager->pixelpitch_align = limits->surface_pixelpitch_alignment;
      manager->bytepitch_align  = limits->surface_bytepitch_alignment;
@@ -142,48 +159,115 @@ dfb_surfacemanager_create( unsigned int     length,
      manager->max_power_of_two_bytepitch  = limits->surface_max_power_of_two_bytepitch;
      manager->max_power_of_two_height     = limits->surface_max_power_of_two_height;
 
-     fusion_skirmish_init( &manager->lock, "Surface Manager" );
-
-     D_MAGIC_SET( chunk, _Chunk_ );
+     fusion_skirmish_init( &manager->lock, "Surface Manager", dfb_core_world(core) );
 
      D_MAGIC_SET( manager, SurfaceManager );
 
      return manager;
 }
 
+DFBResult
+dfb_surfacemanager_add_heap( SurfaceManager     *manager,
+                             CoreSurfaceStorage  storage,
+                             unsigned int        offset,
+                             unsigned int        length )
+{
+     Heap  *heap, *first;
+     Chunk *chunk;
+     
+     D_ASSERT( manager != NULL );
+
+     if (storage == CSS_SYSTEM)
+          return DFB_INVARG;
+
+     heap = SHCALLOC( manager->shmpool, 1, sizeof(Heap) );
+     if (!heap)
+          return D_OOSHM();
+
+     chunk = SHCALLOC( manager->shmpool, 1, sizeof(Chunk) );
+     if (!chunk) {
+          SHFREE( manager->shmpool, heap );
+          return D_OOSHM();
+     }
+
+     D_MAGIC_SET( heap, _Heap_ );
+
+     heap->storage = storage;
+     heap->chunks  = chunk;
+     heap->offset  = offset;
+     heap->length  = length;
+     heap->avail   = length - offset;
+
+     D_MAGIC_SET( chunk, _Chunk_ );
+     
+     chunk->offset = offset;
+     chunk->length = length - offset;
+     chunk->heap   = heap;
+   
+     first = manager->heaps;
+     if (first) {
+          Heap *last = first->prev;
+          heap->prev = last;
+          last->next = first->prev = heap;
+     }
+     else {
+          heap->prev = heap;
+          manager->heaps = heap;
+     }
+
+     D_DEBUG_AT( Core_SM, "Added heap for storage 0x%x (offset %d, length %d).\n",
+                 storage, offset, length );
+     
+     return DFB_OK;
+}
+
 void
 dfb_surfacemanager_destroy( SurfaceManager *manager )
 {
+     Heap  *heap;
      Chunk *chunk;
+     void  *next;
 
      D_ASSERT( manager != NULL );
-     D_ASSERT( manager->chunks != NULL );
 
      D_MAGIC_ASSERT( manager, SurfaceManager );
 
-     D_MAGIC_CLEAR( manager );
+     /* Iterate through heaps. */
+     heap = manager->heaps;
+     while (heap) {
+          /* Deallocate all video chunks. */
+          chunk = heap->chunks;
+          while (chunk) {
+               next = chunk->next;
 
-     /* Deallocate all chunks. */
-     chunk = manager->chunks;
-     while (chunk) {
-          Chunk *next = chunk->next;
+               D_MAGIC_CLEAR( chunk );
 
-          D_MAGIC_CLEAR( chunk );
+               SHFREE( manager->shmpool, chunk );
 
-          SHFREE( chunk );
+               chunk = next;
+          }
 
-          chunk = next;
+          next = heap->next;
+
+          D_MAGIC_CLEAR( heap );
+
+          SHFREE( manager->shmpool, heap );
+
+          heap = next;
      }
 
+     D_MAGIC_CLEAR( manager );
+
      /* Destroy manager lock. */
      fusion_skirmish_destroy( &manager->lock );
 
      /* Deallocate manager struct. */
-     SHFREE( manager );
+     SHFREE( manager->shmpool, manager );
 }
 
 DFBResult dfb_surfacemanager_suspend( SurfaceManager *manager )
 {
+     Heap  *h;
      Chunk *c;
 
      D_MAGIC_ASSERT( manager, SurfaceManager );
@@ -192,17 +276,22 @@ DFBResult dfb_surfacemanager_suspend( SurfaceManager *manager )
 
      manager->suspended = true;
 
-     c = manager->chunks;
-     while (c) {
-          if (c->buffer &&
-              c->buffer->policy != CSP_VIDEOONLY &&
-              c->buffer->video.health == CSH_STORED)
-          {
-               dfb_surfacemanager_assure_system( manager, c->buffer );
-               c->buffer->video.health = CSH_RESTORE;
+     h = manager->heaps;
+     while (h) {
+          c = h->chunks;
+          while (c) {
+               if (c->buffer &&
+                   c->buffer->policy != CSP_VIDEOONLY &&
+                   c->buffer->video.health == CSH_STORED)
+               {
+                    dfb_surfacemanager_assure_system( manager, c->buffer );
+                    c->buffer->video.health = CSH_RESTORE;
+               }
+
+               c = c->next;
           }
 
-          c = c->next;
+          h = h->next;
      }
 
      dfb_surfacemanager_unlock( manager );
@@ -240,7 +329,10 @@ void dfb_surfacemanager_unlock( SurfaceManager *manager )
 DFBResult dfb_surfacemanager_adjust_heap_offset( SurfaceManager *manager,
                                                  int             offset )
 {
+     Heap *heap;
+     
      D_ASSERT( offset >= 0 );
+     D_ASSERT( manager->heaps != NULL );
 
      D_MAGIC_ASSERT( manager, SurfaceManager );
 
@@ -251,12 +343,17 @@ DFBResult dfb_surfacemanager_adjust_heap_offset( SurfaceManager *manager,
           offset -= offset % manager->byteoffset_align;
      }
 
-     if (manager->chunks->buffer == NULL) {
+     /*
+      * Adjust only the offset of the first heap.
+      */
+     heap = manager->heaps;
+     if (heap->chunks->buffer == NULL) {
           /* first chunk is free */
-          if (offset <= manager->chunks->offset + manager->chunks->length) {
+          if (offset <= heap->chunks->offset + heap->chunks->length) {
                /* ok, just recalculate offset and length */
-               manager->chunks->length = manager->chunks->offset + manager->chunks->length - offset;
-               manager->chunks->offset = offset;
+               heap->chunks->length = heap->chunks->offset +
+                                      heap->chunks->length - offset;
+               heap->chunks->offset = offset;
           }
           else {
                D_WARN("unable to adjust heap offset");
@@ -270,8 +367,9 @@ DFBResult dfb_surfacemanager_adjust_heap_offset( SurfaceManager *manager,
           /* TODO: move/destroy instances */
      }
 
-     manager->heap_offset = offset;
-
+     heap->avail -= offset - heap->offset;
+     heap->offset = offset;
+     
      dfb_surfacemanager_unlock( manager );
 
      return DFB_OK;
@@ -282,7 +380,9 @@ dfb_surfacemanager_enumerate_chunks( SurfaceManager  *manager,
                                      SMChunkCallback  callback,
                                      void            *ctx )
 {
+     Heap  *h;
      Chunk *c;
+     DFBEnumerationResult ret = DFENUM_OK;
 
      D_ASSERT( manager != NULL );
      D_ASSERT( callback != NULL );
@@ -291,13 +391,17 @@ dfb_surfacemanager_enumerate_chunks( SurfaceManager  *manager,
 
      dfb_surfacemanager_lock( manager );
 
-     c = manager->chunks;
-     while (c) {
-          if (callback( c->buffer, c->offset,
-                        c->length, c->tolerations, ctx) == DFENUM_CANCEL)
-               break;
+     h = manager->heaps;
+     while (h && ret != DFENUM_CANCEL) {
+          c = h->chunks;
+          while (c && ret != DFENUM_CANCEL) {
+               ret = callback( c->buffer, c->offset,
+                               c->length, c->tolerations, ctx );
 
-          c = c->next;
+               c = c->next;
+          }
+
+          h = h->next;
      }
 
      dfb_surfacemanager_unlock( manager );
@@ -311,16 +415,17 @@ DFBResult dfb_surfacemanager_allocate( SurfaceManager *manager,
 {
      int pitch;
      int length;
+     Heap  *h;
      Chunk *c;
 
      Chunk *best_free = NULL;
      Chunk *best_occupied = NULL;
-
+     
      CoreSurface *surface = buffer->surface;
 
      D_MAGIC_ASSERT( manager, SurfaceManager );
 
-     if (!manager->length || manager->suspended)
+     if (manager->suspended)
           return DFB_NOVIDEOMEMORY;
 
      /* calculate the required length depending on limitations */
@@ -346,6 +451,8 @@ DFBResult dfb_surfacemanager_allocate( SurfaceManager *manager,
           pitch -= pitch % manager->bytepitch_align;
      }
 
+     buffer->video.pitch = pitch;
+
      length = DFB_PLANE_MULTIPLY( buffer->format,
                                   MAX( surface->height, surface->min_height ) * pitch );
 
@@ -354,77 +461,78 @@ DFBResult dfb_surfacemanager_allocate( SurfaceManager *manager,
           length -= length % manager->byteoffset_align;
      }
 
-     /* Do a pre check before iterating through all chunks. */
-     if (length > manager->available - manager->heap_offset)
-          return DFB_NOVIDEOMEMORY;
+     /* iterate through heaps */
+     for (h = manager->heaps; h; h = h->next) { 
+          if (length > h->avail)
+               continue;
 
-     buffer->video.pitch = pitch;
+          /* examine chunks */
+          c = h->chunks;
+          while (c) {
+               if (c->length >= length) {
+                    if (c->buffer) {
+                         c->tolerations++;
+                         if (c->tolerations > 0xff)
+                              c->tolerations = 0xff;
 
-     /* examine chunks */
-     c = manager->chunks;
-     while (c) {
-          if (c->length >= length) {
-               if (c->buffer) {
-                    c->tolerations++;
-                    if (c->tolerations > 0xff)
-                         c->tolerations = 0xff;
-
-                    if (!c->buffer->video.locked              &&
-                        c->buffer->policy <= buffer->policy   &&
-                        c->buffer->policy != CSP_VIDEOONLY    &&
-
-                        ((buffer->policy > c->buffer->policy) ||
-                         (c->tolerations > manager->min_toleration/8 + 2)))
-                    {
+                         if (!c->buffer->video.locked              &&
+                             c->buffer->policy <= buffer->policy   &&
+                             c->buffer->policy != CSP_VIDEOONLY    &&
+                            ((buffer->policy > c->buffer->policy)  ||
+                             (c->tolerations > h->min_toleration/8 + 2)))
+                         {
+                              /* found a nice place to chill */
+                              if (!best_occupied  ||
+                                  best_occupied->length > c->length  ||
+                                  best_occupied->tolerations < c->tolerations)
+                                   /* first found or better one? */
+                                   best_occupied = c;
+                         }
+                    }
+                    else {
                          /* found a nice place to chill */
-                         if (!best_occupied  ||
-                              best_occupied->length > c->length  ||
-                              best_occupied->tolerations < c->tolerations)
+                         if (!best_free  ||  best_free->length > c->length)
                               /* first found or better one? */
-                              best_occupied = c;
+                              best_free = c;
                     }
                }
-               else {
-                    /* found a nice place to chill */
-                    if (!best_free  ||  best_free->length > c->length)
-                         /* first found or better one? */
-                         best_free = c;
-               }
+
+               c = c->next;
           }
 
-          c = c->next;
-     }
+          /* if we found a place */
+          if (best_free) {
+               occupy_chunk( manager, best_free, buffer, length );
 
-     /* if we found a place */
-     if (best_free) {
-          occupy_chunk( manager, best_free, buffer, length );
-
-          return DFB_OK;
+               return DFB_OK;
+          }
      }
 
      if (best_occupied) {
-          CoreSurface *kicked = best_occupied->buffer->surface;
+          SurfaceBuffer *kicked = best_occupied->buffer;
 
-          D_DEBUG_AT( Core_SM, "Kicking out buffer at %d (%d) with tolerations %d...\n",
+          D_DEBUG_AT( Core_SM,
+                      "Kicking out buffer at %d (%d) with tolerations %d...\n",
                       best_occupied->offset,
                       best_occupied->length, best_occupied->tolerations );
 
-          dfb_surfacemanager_assure_system( manager, best_occupied->buffer );
+          dfb_surfacemanager_assure_system( manager, kicked );
 
-          best_occupied->buffer->video.health = CSH_INVALID;
-          dfb_surface_notify_listeners( kicked, CSNF_VIDEO );
+          kicked->video.health = CSH_INVALID;
+          dfb_surface_notify_listeners( kicked->surface, CSNF_VIDEO );
 
           best_occupied = free_chunk( manager, best_occupied );
 
-          dfb_gfxcard_sync();
-
+          if (kicked->video.access & VAF_HARDWARE_READ) {
+               dfb_gfxcard_sync();
+               kicked->video.access &= ~VAF_HARDWARE_READ;
+          }
 
           occupy_chunk( manager, best_occupied, buffer, length );
 
           return DFB_OK;
      }
 
-
      D_DEBUG_AT( Core_SM, "Couldn't allocate enough heap space for video memory surface!\n" );
 
      /* no luck */
@@ -446,6 +554,7 @@ DFBResult dfb_surfacemanager_deallocate( SurfaceManager *manager,
 
      buffer->video.health = CSH_INVALID;
      buffer->video.chunk = NULL;
+     buffer->storage = 0;
 
      dfb_surface_notify_listeners( buffer->surface, CSNF_VIDEO );
 
@@ -478,9 +587,13 @@ DFBResult dfb_surfacemanager_assure_video( SurfaceManager *manager,
 
      switch (buffer->video.health) {
           case CSH_STORED:
-               if (buffer->video.chunk)
+               if (buffer->video.chunk) {
                     buffer->video.chunk->tolerations = 0;
-
+                    buffer->storage = buffer->video.chunk->heap->storage;
+               }
+               else {
+                    buffer->storage = CSS_VIDEO;
+               }
                return DFB_OK;
 
           case CSH_INVALID:
@@ -497,9 +610,15 @@ DFBResult dfb_surfacemanager_assure_video( SurfaceManager *manager,
 
                if (buffer->flags & SBF_WRITTEN) {
                     int   i;
-                    char *src = buffer->system.addr;
-                    char *dst = dfb_system_video_memory_virtual( buffer->video.offset );
-
+                    char *src   = buffer->system.addr;
+                    char *dst   = (buffer->video.chunk->heap->storage == CSS_VIDEO)
+                                  ? dfb_system_video_memory_virtual( buffer->video.offset )
+                                  : dfb_system_aux_memory_virtual( buffer->video.offset );
+                    int   flags = buffer->video.access & (VAF_SOFTWARE_LOCK | VAF_SOFTWARE_WRITE);
+                   
+                    if (flags != (VAF_SOFTWARE_LOCK | VAF_SOFTWARE_WRITE))
+                         dfb_gfxcard_surface_enter( buffer, DSLF_WRITE );
+                    
                     for (i=0; i<surface->height; i++) {
                          direct_memcpy( dst, src,
                                         DFB_BYTES_PER_LINE(buffer->format, surface->width) );
@@ -531,10 +650,15 @@ DFBResult dfb_surfacemanager_assure_video( SurfaceManager *manager,
                               dst += buffer->video.pitch;
                          }
                     }
+
+                    if (flags != (VAF_SOFTWARE_LOCK | VAF_SOFTWARE_WRITE))
+                         dfb_gfxcard_surface_leave( buffer );
                }
 
                buffer->video.health             = CSH_STORED;
                buffer->video.chunk->tolerations = 0;
+               
+               buffer->storage = buffer->video.chunk->heap->storage;
 
                dfb_surface_notify_listeners( surface, CSNF_VIDEO );
 
@@ -560,20 +684,29 @@ DFBResult dfb_surfacemanager_assure_system( SurfaceManager *manager,
           return DFB_BUG;
      }
 
-     if (buffer->system.health == CSH_STORED)
+     if (buffer->system.health == CSH_STORED) {
+          buffer->storage = CSS_SYSTEM;
           return DFB_OK;
+     }
      else if (buffer->video.health == CSH_STORED) {
           int   i;
-          char *src = dfb_system_video_memory_virtual( buffer->video.offset );
-          char *dst = buffer->system.addr;
-
+          char *src   = (buffer->video.chunk->heap->storage == CSS_VIDEO)
+                        ? dfb_system_video_memory_virtual( buffer->video.offset )
+                        : dfb_system_aux_memory_virtual( buffer->video.offset );
+          char *dst   = buffer->system.addr;
+          int   flags = buffer->video.access & (VAF_SOFTWARE_LOCK | VAF_SOFTWARE_READ);
+           
           /* from video_access_by_software() in surface.c */
           if (buffer->video.access & VAF_HARDWARE_WRITE) {
-               dfb_gfxcard_sync();
+               dfb_gfxcard_wait_serial( &buffer->video.serial );
+               dfb_gfxcard_flush_read_cache();
                buffer->video.access &= ~VAF_HARDWARE_WRITE;
           }
           buffer->video.access |= VAF_SOFTWARE_READ;
 
+          if (flags != (VAF_SOFTWARE_LOCK | VAF_SOFTWARE_READ))
+               dfb_gfxcard_surface_enter( buffer, DSLF_READ );
+
           for (i=0; i<surface->height; i++) {
                direct_memcpy( dst, src, DFB_BYTES_PER_LINE(buffer->format, surface->width) );
                src += buffer->video.pitch;
@@ -605,8 +738,13 @@ DFBResult dfb_surfacemanager_assure_system( SurfaceManager *manager,
                }
           }
 
+          if (flags != (VAF_SOFTWARE_LOCK | VAF_SOFTWARE_READ))
+               dfb_gfxcard_surface_leave( buffer );
+
           buffer->system.health = CSH_STORED;
 
+          buffer->storage = CSS_SYSTEM;
+
           dfb_surface_notify_listeners( surface, CSNF_SYSTEM );
 
           return DFB_OK;
@@ -618,20 +756,23 @@ DFBResult dfb_surfacemanager_assure_system( SurfaceManager *manager,
 
 /** internal functions NOT locking the surfacemanager **/
 
-static Chunk* split_chunk( Chunk *c, int length )
+static Chunk *
+split_chunk( SurfaceManager *manager, Chunk *c, int length )
 {
      Chunk *newchunk;
 
      D_MAGIC_ASSERT( c, _Chunk_ );
+     D_MAGIC_ASSERT( c->heap, _Heap_ );
 
      if (c->length == length)          /* does not need be splitted */
           return c;
 
-     newchunk = (Chunk*) SHCALLOC( 1, sizeof(Chunk) );
+     newchunk = (Chunk*) SHCALLOC( manager->shmpool, 1, sizeof(Chunk) );
 
      /* calculate offsets and lengths of resulting chunks */
      newchunk->offset = c->offset + c->length - length;
      newchunk->length = length;
+     newchunk->heap   = c->heap;
      c->length -= newchunk->length;
 
      /* insert newchunk after chunk c */
@@ -646,27 +787,29 @@ static Chunk* split_chunk( Chunk *c, int length )
      return newchunk;
 }
 
-static Chunk*
+static Chunk *
 free_chunk( SurfaceManager *manager, Chunk *chunk )
 {
      D_MAGIC_ASSERT( manager, SurfaceManager );
      D_MAGIC_ASSERT( chunk, _Chunk_ );
+     D_MAGIC_ASSERT( chunk->heap, _Heap_ );
 
      if (!chunk->buffer) {
           D_BUG( "freeing free chunk" );
           return chunk;
      }
      else {
-          D_DEBUG_AT( Core_SM, "Deallocating %d bytes at offset %d.\n",
-                      chunk->length, chunk->offset );
+          D_DEBUG_AT( Core_SM, 
+                      "Deallocating %d bytes at offset %d (storage 0x%x).\n",
+                      chunk->length, chunk->offset, chunk->heap->storage );
      }
 
      if (chunk->buffer->policy == CSP_VIDEOONLY)
-          manager->available += chunk->length;
+          chunk->heap->avail += chunk->length;
 
      chunk->buffer = NULL;
 
-     manager->min_toleration--;
+     chunk->heap->min_toleration--;
 
      if (chunk->prev  &&  !chunk->prev->buffer) {
           Chunk *prev = chunk->prev;
@@ -683,7 +826,7 @@ free_chunk( SurfaceManager *manager, Chunk *chunk )
 
           D_MAGIC_CLEAR( chunk );
 
-          SHFREE( chunk );
+          SHFREE( manager->shmpool, chunk );
           chunk = prev;
      }
 
@@ -700,7 +843,7 @@ free_chunk( SurfaceManager *manager, Chunk *chunk )
 
           D_MAGIC_CLEAR( next );
 
-          SHFREE( next );
+          SHFREE( manager->shmpool, next );
      }
 
      return chunk;
@@ -711,20 +854,23 @@ occupy_chunk( SurfaceManager *manager, Chunk *chunk, SurfaceBuffer *buffer, int
 {
      D_MAGIC_ASSERT( manager, SurfaceManager );
      D_MAGIC_ASSERT( chunk, _Chunk_ );
-
+     D_MAGIC_ASSERT( chunk->heap, _Heap_ );
+     
      if (buffer->policy == CSP_VIDEOONLY)
-          manager->available -= length;
+          chunk->heap->avail -= length;
 
-     chunk = split_chunk( chunk, length );
+     chunk = split_chunk( manager, chunk, length );
 
-     D_DEBUG_AT( Core_SM, "Allocating %d bytes at offset %d.\n", chunk->length, chunk->offset );
+     D_DEBUG_AT( Core_SM, "Allocating %d bytes at offset %d (storage 0x%x).\n",
+                 chunk->length, chunk->offset, chunk->heap->storage );
 
+     buffer->storage      = chunk->heap->storage;
      buffer->video.health = CSH_RESTORE;
      buffer->video.offset = chunk->offset;
      buffer->video.chunk  = chunk;
 
      chunk->buffer = buffer;
 
-     manager->min_toleration++;
+     chunk->heap->min_toleration++;
 }
 
