@@ -81,6 +81,15 @@ typedef enum {
                                            store in video memory */
 } CoreSurfacePolicy;
 
+typedef enum {
+     CSS_SYSTEM          = 0x00000000,  /* actually stored 
+                                           into system memory */ 
+     CSS_VIDEO           = 0x00000001,  /* actually stored 
+                                           into video memory */
+     CSS_AUXILIARY       = 0x00000002,  /* actually stored 
+                                           into PCI/AGP/PCIE memory */
+} CoreSurfaceStorage;
+
 typedef enum {
      SBF_NONE            = 0x00000000,
      SBF_FOREIGN_SYSTEM  = 0x00000001,  /* system memory is preallocated by
@@ -95,7 +104,8 @@ typedef enum {
      VAF_SOFTWARE_WRITE  = 0x00000001,  /* software wrote to buffer */
      VAF_HARDWARE_WRITE  = 0x00000002,  /* hardware wrote to buffer */
      VAF_SOFTWARE_READ   = 0x00000004,  /* software read from buffer */
-     VAF_HARDWARE_READ   = 0x00000008   /* hardware read from buffer */
+     VAF_HARDWARE_READ   = 0x00000008,  /* hardware read from buffer */
+     VAF_SOFTWARE_LOCK   = 0x00000010   /* software access to buffer */
 } VideoAccessFlags;
 
 
@@ -103,6 +113,7 @@ struct _SurfaceBuffer
 {
      SurfaceBufferFlags      flags;     /* additional information */
      CoreSurfacePolicy       policy;    /* swapping policy for surfacemanager */
+     CoreSurfaceStorage      storage;   /* system, video or bus memory? */
 
      DFBSurfacePixelFormat   format;    /* pixel format of buffer, usually that of its surface */
 
@@ -119,7 +130,7 @@ struct _SurfaceBuffer
           CoreSurfaceHealth  health;    /* currently stored in video memory? */
           int                locked;    /* video instance is locked, don't
                                            try to kick out, could deadlock */
-
+          
           VideoAccessFlags   access;    /* information about recent read/write
                                            accesses to video buffer memory */
           CoreGraphicsSerial serial;    /* serial of last write operation */
@@ -168,6 +179,9 @@ struct _CoreSurface
      SurfaceBuffer         *depth_buffer;  /* Z buffer for 3D rendering */
 
      SurfaceManager        *manager;
+
+     FusionSHMPoolShared   *shmpool;       /* for structures, e.g. CoreSurface or SurfaceBuffer */
+     FusionSHMPoolShared   *shmpool_data;  /* for raw data in the surface buffers */
 };
 
 static inline void *
@@ -198,7 +212,7 @@ dfb_surface_data_offset( const CoreSurface *surface,
 /*
  * Creates a pool of surface objects.
  */
-FusionObjectPool *dfb_surface_pool_create();
+FusionObjectPool *dfb_surface_pool_create( const FusionWorld *world );
 
 /*
  * Generates dfb_surface_ref(), dfb_surface_attach() etc.
