@@ -66,14 +66,24 @@
 #include <gfx/util.h>
 
 
-static ReactionResult
-IDirectFBSurface_listener( const void *msg_data, void *ctx );
+D_DEBUG_DOMAIN( Surface, "IDirectFBSurface", "IDirectFBSurface Interface" );
 
+/**********************************************************************************************************************/
+
+static ReactionResult IDirectFBSurface_listener( const void *msg_data, void *ctx );
+
+/**********************************************************************************************************************/
 
 void
 IDirectFBSurface_Destruct( IDirectFBSurface *thiz )
 {
-     IDirectFBSurface_data *data = (IDirectFBSurface_data*)thiz->priv;
+     IDirectFBSurface_data *data;
+
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
+     D_ASSERT( thiz != NULL );
+
+     data = thiz->priv;
 
      if (data->surface)
           dfb_surface_detach( data->surface, &data->reaction );
@@ -97,8 +107,12 @@ IDirectFBSurface_Destruct( IDirectFBSurface *thiz )
                D_WARN( "font dead?" );
      }
 
-     if (data->surface)
+     if (data->surface) {
+          if (data->locked)
+               dfb_surface_unlock( data->surface, data->locked - 1 );
+
           dfb_surface_unref( data->surface );
+     }
 
      DIRECT_DEALLOCATE_INTERFACE( thiz );
 }
@@ -108,6 +122,8 @@ IDirectFBSurface_AddRef( IDirectFBSurface *thiz )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
      data->ref++;
 
      return DFB_OK;
@@ -118,6 +134,8 @@ IDirectFBSurface_Release( IDirectFBSurface *thiz )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
      if (--data->ref == 0)
           IDirectFBSurface_Destruct( thiz );
 
@@ -131,6 +149,8 @@ IDirectFBSurface_GetPixelFormat( IDirectFBSurface      *thiz,
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
      if (!data->surface)
           return DFB_DESTROYED;
 
@@ -149,6 +169,8 @@ IDirectFBSurface_GetAccelerationMask( IDirectFBSurface    *thiz,
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
      if (!data->surface)
           return DFB_DESTROYED;
 
@@ -188,6 +210,8 @@ IDirectFBSurface_GetSize( IDirectFBSurface *thiz,
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
      if (!width && !height)
           return DFB_INVARG;
 
@@ -206,6 +230,8 @@ IDirectFBSurface_GetVisibleRectangle( IDirectFBSurface *thiz,
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
      if (!rect)
           return DFB_INVARG;
 
@@ -223,6 +249,8 @@ IDirectFBSurface_GetCapabilities( IDirectFBSurface       *thiz,
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
      if (!caps)
           return DFB_INVARG;
 
@@ -241,6 +269,8 @@ IDirectFBSurface_GetPalette( IDirectFBSurface  *thiz,
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
      surface = data->surface;
      if (!surface)
           return DFB_DESTROYED;
@@ -271,6 +301,8 @@ IDirectFBSurface_SetPalette( IDirectFBSurface *thiz,
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
      surface = data->surface;
      if (!surface)
           return DFB_DESTROYED;
@@ -302,6 +334,8 @@ IDirectFBSurface_SetAlphaRamp( IDirectFBSurface *thiz,
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
      if (!data->surface)
           return DFB_DESTROYED;
 
@@ -322,9 +356,13 @@ IDirectFBSurface_Lock( IDirectFBSurface *thiz,
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
      if (!data->surface)
           return DFB_DESTROYED;
 
+     if (data->locked)
+          return DFB_LOCKED;
 
      if (!flags || !ret_ptr || !ret_pitch)
           return DFB_INVARG;
@@ -354,6 +392,8 @@ IDirectFBSurface_Unlock( IDirectFBSurface *thiz )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
      if (data->locked)
           dfb_surface_unlock( data->surface, data->locked - 1 );
 
@@ -372,6 +412,8 @@ IDirectFBSurface_Flip( IDirectFBSurface    *thiz,
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
+     D_DEBUG_AT( Surface, "%s( %p, %p, 0x%08x )\n", __FUNCTION__, thiz, region, flags );
+
      surface = data->surface;
      if (!surface)
           return DFB_DESTROYED;
@@ -398,6 +440,8 @@ IDirectFBSurface_Flip( IDirectFBSurface    *thiz,
                return DFB_INVAREA;
      }
 
+     D_DEBUG_AT( Surface, "  -> %d, %d - %dx%d\n", DFB_RECTANGLE_VALS_FROM_REGION( &reg ) );
+
      if (!(flags & DSFLIP_BLIT) && reg.x1 == 0 && reg.y1 == 0 &&
          reg.x2 == surface->width - 1 && reg.y2 == surface->height - 1)
           dfb_surface_flip_buffers( data->surface, false );
@@ -413,6 +457,8 @@ IDirectFBSurface_SetField( IDirectFBSurface    *thiz,
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
      if (!data->surface)
           return DFB_DESTROYED;
 
@@ -439,6 +485,8 @@ IDirectFBSurface_Clear( IDirectFBSurface *thiz,
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
      surface = data->surface;
      if (!surface)
           return DFB_DESTROYED;
@@ -490,6 +538,8 @@ IDirectFBSurface_SetClip( IDirectFBSurface *thiz, const DFBRegion *clip )
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
 
      if (!data->area.current.w || !data->area.current.h)
           return DFB_INVAREA;
@@ -517,6 +567,24 @@ IDirectFBSurface_SetClip( IDirectFBSurface *thiz, const DFBRegion *clip )
      return DFB_OK;
 }
 
+static DFBResult
+IDirectFBSurface_GetClip( IDirectFBSurface *thiz, DFBRegion *ret_clip )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
+
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
+     if (!ret_clip)
+          return DFB_INVARG;
+
+     if (!data->area.current.w || !data->area.current.h)
+          return DFB_INVAREA;
+
+     *ret_clip = DFB_REGION_INIT_TRANSLATED( &data->state.clip, -data->area.wanted.x, -data->area.wanted.y );
+
+     return DFB_OK;
+}
+
 static DFBResult
 IDirectFBSurface_SetColor( IDirectFBSurface *thiz,
                            __u8 r, __u8 g, __u8 b, __u8 a )
@@ -526,6 +594,8 @@ IDirectFBSurface_SetColor( IDirectFBSurface *thiz,
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
      surface = data->surface;
      if (!surface)
           return DFB_DESTROYED;
@@ -548,6 +618,8 @@ IDirectFBSurface_SetColorIndex( IDirectFBSurface *thiz,
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
      surface = data->surface;
      if (!surface)
           return DFB_DESTROYED;
@@ -575,6 +647,8 @@ IDirectFBSurface_SetSrcBlendFunction( IDirectFBSurface        *thiz,
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
      switch (src) {
           case DSBF_ZERO:
           case DSBF_ONE:
@@ -600,6 +674,8 @@ IDirectFBSurface_SetDstBlendFunction( IDirectFBSurface        *thiz,
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
      switch (dst) {
           case DSBF_ZERO:
           case DSBF_ONE:
@@ -628,6 +704,8 @@ IDirectFBSurface_SetPorterDuff( IDirectFBSurface         *thiz,
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
 
      switch (rule) {
           case DSPD_NONE:
@@ -686,6 +764,8 @@ IDirectFBSurface_SetSrcColorKey( IDirectFBSurface *thiz,
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
      surface = data->surface;
      if (!surface)
           return DFB_DESTROYED;
@@ -716,6 +796,8 @@ IDirectFBSurface_SetSrcColorKeyIndex( IDirectFBSurface *thiz,
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
      surface = data->surface;
      if (!surface)
           return DFB_DESTROYED;
@@ -753,6 +835,8 @@ IDirectFBSurface_SetDstColorKey( IDirectFBSurface *thiz,
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
      surface = data->surface;
      if (!surface)
           return DFB_DESTROYED;
@@ -781,6 +865,8 @@ IDirectFBSurface_SetDstColorKeyIndex( IDirectFBSurface *thiz,
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
      surface = data->surface;
      if (!surface)
           return DFB_DESTROYED;
@@ -814,6 +900,8 @@ IDirectFBSurface_SetFont( IDirectFBSurface *thiz,
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
      if (data->font != font) {
          if (font) {
               ret = font->AddRef( font );
@@ -844,6 +932,8 @@ IDirectFBSurface_GetFont( IDirectFBSurface  *thiz,
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
      if (!font)
           return DFB_INVARG;
 
@@ -864,6 +954,8 @@ IDirectFBSurface_SetDrawingFlags( IDirectFBSurface       *thiz,
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
      dfb_state_set_drawing_flags( &data->state, flags );
 
      return DFB_OK;
@@ -877,6 +969,8 @@ IDirectFBSurface_FillRectangle( IDirectFBSurface *thiz,
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
      if (!data->surface)
           return DFB_DESTROYED;
 
@@ -907,6 +1001,8 @@ IDirectFBSurface_DrawLine( IDirectFBSurface *thiz,
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
      if (!data->surface)
           return DFB_DESTROYED;
 
@@ -936,6 +1032,8 @@ IDirectFBSurface_DrawLines( IDirectFBSurface *thiz,
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
      if (!data->surface)
           return DFB_DESTROYED;
 
@@ -976,6 +1074,8 @@ IDirectFBSurface_DrawRectangle( IDirectFBSurface *thiz,
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
      if (!data->surface)
           return DFB_DESTROYED;
 
@@ -1007,6 +1107,8 @@ IDirectFBSurface_FillTriangle( IDirectFBSurface *thiz,
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
      if (!data->surface)
           return DFB_DESTROYED;
 
@@ -1036,6 +1138,8 @@ IDirectFBSurface_FillRectangles( IDirectFBSurface   *thiz,
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
      if (!data->surface)
           return DFB_DESTROYED;
 
@@ -1087,6 +1191,8 @@ IDirectFBSurface_FillSpans( IDirectFBSurface *thiz,
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
      if (!data->surface)
           return DFB_DESTROYED;
 
@@ -1123,6 +1229,8 @@ IDirectFBSurface_SetBlittingFlags( IDirectFBSurface        *thiz,
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
      dfb_state_set_blitting_flags( &data->state, flags );
 
      return DFB_OK;
@@ -1139,6 +1247,8 @@ IDirectFBSurface_Blit( IDirectFBSurface   *thiz,
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
      if (!data->surface)
           return DFB_DESTROYED;
 
@@ -1204,6 +1314,8 @@ IDirectFBSurface_TileBlit( IDirectFBSurface   *thiz,
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
      if (!data->surface)
           return DFB_DESTROYED;
 
@@ -1284,6 +1396,8 @@ IDirectFBSurface_BatchBlit( IDirectFBSurface   *thiz,
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
      if (!data->surface)
           return DFB_DESTROYED;
 
@@ -1350,6 +1464,8 @@ IDirectFBSurface_StretchBlit( IDirectFBSurface   *thiz,
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
      if (!data->surface)
           return DFB_DESTROYED;
 
@@ -1456,6 +1572,8 @@ IDirectFBSurface_TextureTriangles( IDirectFBSurface     *thiz,
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
      if (!data->surface)
           return DFB_DESTROYED;
 
@@ -1555,6 +1673,8 @@ IDirectFBSurface_DrawString( IDirectFBSurface *thiz,
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
      if (!data->surface)
           return DFB_DESTROYED;
 
@@ -1622,6 +1742,8 @@ IDirectFBSurface_DrawGlyph( IDirectFBSurface *thiz,
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
      if (!data->surface)
           return DFB_DESTROYED;
 
@@ -1682,6 +1804,8 @@ IDirectFBSurface_GetSubSurface( IDirectFBSurface    *thiz,
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
      /* Check arguments */
      if (!data->surface)
           return DFB_DESTROYED;
@@ -1726,6 +1850,8 @@ IDirectFBSurface_GetGL( IDirectFBSurface   *thiz,
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
      if (!data->surface)
           return DFB_DESTROYED;
 
@@ -1760,6 +1886,8 @@ IDirectFBSurface_Dump( IDirectFBSurface   *thiz,
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
      if (!directory || !prefix)
           return DFB_INVARG;
 
@@ -1784,6 +1912,8 @@ IDirectFBSurface_DisableAcceleration( IDirectFBSurface    *thiz,
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
      if (D_FLAGS_INVALID( mask, DFXL_ALL ))
           return DFB_INVARG;
 
@@ -1804,6 +1934,8 @@ DFBResult IDirectFBSurface_Construct( IDirectFBSurface       *thiz,
 
      DIRECT_ALLOCATE_INTERFACE_DATA(thiz, IDirectFBSurface)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
      data->ref = 1;
      data->caps = caps | surface->caps;
 
@@ -1830,7 +1962,7 @@ DFBResult IDirectFBSurface_Construct( IDirectFBSurface       *thiz,
 
      data->surface = surface;
 
-     dfb_state_init( &data->state );
+     dfb_state_init( &data->state, NULL );
      dfb_state_set_destination( &data->state, surface );
 
      data->state.clip.x1  = data->area.current.x;
@@ -1860,6 +1992,7 @@ DFBResult IDirectFBSurface_Construct( IDirectFBSurface       *thiz,
      thiz->Clear = IDirectFBSurface_Clear;
 
      thiz->SetClip = IDirectFBSurface_SetClip;
+     thiz->GetClip = IDirectFBSurface_GetClip;
      thiz->SetColor = IDirectFBSurface_SetColor;
      thiz->SetColorIndex = IDirectFBSurface_SetColorIndex;
      thiz->SetSrcBlendFunction = IDirectFBSurface_SetSrcBlendFunction;
@@ -1916,6 +2049,8 @@ IDirectFBSurface_listener( const void *msg_data, void *ctx )
      IDirectFBSurface_data         *data         = thiz->priv;
      CoreSurface                   *surface      = data->surface;
 
+     D_DEBUG_AT( Surface, "%s( %p, %p )\n", __FUNCTION__, msg_data, ctx );
+
      if (notification->flags & CSNF_DESTROY) {
           if (data->surface) {
                data->surface = NULL;
