@@ -55,8 +55,13 @@
 
 #include "gfx/util.h"
 
+
+D_DEBUG_DOMAIN( Surface, "IDirectFBSurfaceW", "IDirectFBSurface_Window Interface" );
+
+/**********************************************************************************************************************/
+
 /*
- * private data struct of IDirectFB
+ * private data struct of IDirectFBSurface_Window
  */
 typedef struct {
      IDirectFBSurface_data base;   /* base Surface implementation */
@@ -69,16 +74,22 @@ typedef struct {
 //     CoreGraphicsSerial    serial;
 } IDirectFBSurface_Window_data;
 
+/**********************************************************************************************************************/
 
-static void *
-Flipping_Thread( void *arg );
+static void *Flipping_Thread( void *arg );
 
+/**********************************************************************************************************************/
 
 static void
 IDirectFBSurface_Window_Destruct( IDirectFBSurface *thiz )
 {
-     IDirectFBSurface_Window_data *data =
-          (IDirectFBSurface_Window_data*)thiz->priv;
+     IDirectFBSurface_Window_data *data;
+
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
+     D_ASSERT( thiz != NULL );
+
+     data = thiz->priv;
 
      if ((int) data->flip_thread != -1) {
           pthread_cancel( data->flip_thread );
@@ -95,6 +106,8 @@ IDirectFBSurface_Window_Release( IDirectFBSurface *thiz )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Window)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
      if (--data->base.ref == 0)
           IDirectFBSurface_Window_Destruct( thiz );
 
@@ -110,6 +123,8 @@ IDirectFBSurface_Window_Flip( IDirectFBSurface    *thiz,
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Window)
 
+     D_DEBUG_AT( Surface, "%s( %p, %p, 0x%08x )\n", __FUNCTION__, thiz, region, flags );
+
      if (!data->base.surface)
           return DFB_DESTROYED;
 
@@ -132,6 +147,8 @@ IDirectFBSurface_Window_Flip( IDirectFBSurface    *thiz,
                return DFB_INVAREA;
      }
 
+     D_DEBUG_AT( Surface, "  -> %d, %d - %dx%d\n", DFB_RECTANGLE_VALS_FROM_REGION( &reg ) );
+
 
      if (flags & DSFLIP_PIPELINE) {
           dfb_gfxcard_wait_serial( &data->window->serial2 );
@@ -141,6 +158,7 @@ IDirectFBSurface_Window_Flip( IDirectFBSurface    *thiz,
           dfb_state_get_serial( &data->base.state, &data->window->serial1 );
      }
 
+
      if (data->window->region) {
           dfb_layer_region_flip_update( data->window->region, &reg, flags );
      }
@@ -172,6 +190,8 @@ IDirectFBSurface_Window_GetSubSurface( IDirectFBSurface    *thiz,
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Window)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
      /* Check arguments */
      if (!data->base.surface || !data->window || !data->window->surface)
           return DFB_DESTROYED;
@@ -218,6 +238,8 @@ IDirectFBSurface_Window_Construct( IDirectFBSurface       *thiz,
 
      DIRECT_ALLOCATE_INTERFACE_DATA(thiz, IDirectFBSurface_Window)
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
      ret = IDirectFBSurface_Construct( thiz, wanted, granted,
                                        window->surface, caps );
      if (ret)
@@ -256,6 +278,8 @@ Flipping_Thread( void *arg )
      IDirectFBSurface             *thiz = (IDirectFBSurface*) arg;
      IDirectFBSurface_Window_data *data = (IDirectFBSurface_Window_data*) thiz->priv;
 
+     D_DEBUG_AT( Surface, "%s( %p )\n", __FUNCTION__, thiz );
+
      while (data->base.surface && data->window->surface) {
           pthread_testcancel();
 
