@@ -78,13 +78,16 @@ static const char *config_usage =
      "  [no-]trace                     Enable stack trace support\n"
      "  log-file=<name>                Write all messages to a file\n"
      "  log-udp=<host>:<port>          Send all messages via UDP to host:port\n"
+     "  fatal-level=<level>            Abort on NONE, ASSERT (default) or ASSUME (incl. assert)\n"
      "  force-windowed                 Primary surface always is a window\n"
      "  force-desktop                  Primary surface is the desktop background\n"
      "  [no-]hardware                  Hardware acceleration\n"
+     "  [no-]dma                       Enable DMA acceleration\n"
      "  [no-]sync                      Do `sync()' (default=no)\n"
 #ifdef USE_MMX
      "  [no-]mmx                       Enable mmx support\n"
 #endif
+     "  [no-]agp[=<mode>]              Enable AGP support\n"
      "  font-format=<pixelformat>      Set the preferred font format\n"
      "  dont-catch=<num>[[,<num>]...]  Don't catch these signals\n"
      "  [no-]sighandler                Enable signal handler\n"
@@ -110,6 +113,7 @@ static const char *config_usage =
      "  [no-]translucent-windows       Allow translucent windows\n"
      "  [no-]decorations               Enable window decorations (if supported by wm)\n"
      "  videoram-limit=<amount>        Limit amount of Video RAM in kb\n"
+     "  agpmem-limit=<amount>          Limit amount of AGP memory in kb\n" 
      "  screenshot-dir=<directory>     Dump screen content on <Print> key presses\n"
      "  disable-module=<module_name>   suppress loading this module\n"
      "\n"
@@ -120,6 +124,8 @@ static const char *config_usage =
      "                                 Matrox cable type (default=composite)\n"
      "  h3600-device=<device>          Use this device for the H3600 TS driver\n"
      "  mut-device=<device>            Use this device for the MuTouch driver\n"
+     "  penmount-device=<device>       Use this device for the PenMount driver\n"
+     "  unichrome-revision=<rev>       Override unichrome hardware revision\n"
      "\n"
      " Window surface swapping policy:\n"
      "  window-surface-policy=(auto|videohigh|videolow|systemonly|videoonly)\n"
@@ -155,6 +161,7 @@ static const FormatString format_strings[] = {
      { "ARGB1555", DSPF_ARGB1555 },
      { "ARGB2554", DSPF_ARGB2554 },
      { "ARGB4444", DSPF_ARGB4444 },
+     { "AYUV",     DSPF_AYUV     },
      { "AiRGB",    DSPF_AiRGB    },
      { "I420",     DSPF_I420     },
      { "LUT8",     DSPF_LUT8     },
@@ -328,7 +335,10 @@ static void config_allocate()
      dfb_config->buffer_mode              = -1;
      dfb_config->wm                       = D_STRDUP( "default" );
      dfb_config->decorations              = true;
-
+     dfb_config->unichrome_revision       = -1;
+     dfb_config->dma                      = false;
+     dfb_config->agp                      = 0;
+     
      /* default to fbdev */
      dfb_config->system = D_STRDUP( "FBDev" );
 
@@ -634,6 +644,21 @@ DFBResult dfb_config_set( const char *name, const char *value )
                return DFB_INVARG;
           }
      } else
+     if (strcmp (name, "fatal-level" ) == 0) {
+          if (strcasecmp (value, "none" ) == 0) {
+               direct_config->fatal = DCFL_NONE;
+          } else
+          if (strcasecmp (value, "assert" ) == 0) {
+               direct_config->fatal = DCFL_ASSERT;
+          } else
+          if (strcasecmp (value, "assume" ) == 0) {
+               direct_config->fatal = DCFL_ASSUME;
+          }
+          else {
+               D_ERROR("DirectFB/Config 'fatal-level': Unknown level specified (use 'none', 'assert', 'assume')!\n");
+               return DFB_INVARG;
+          }
+     } else
      if (strcmp (name, "force-windowed" ) == 0) {
           dfb_config->force_windowed = true;
      } else
@@ -646,12 +671,57 @@ DFBResult dfb_config_set( const char *name, const char *value )
      if (strcmp (name, "no-hardware" ) == 0) {
           dfb_config->software_only = true;
      } else
+     if (strcmp (name, "dma" ) == 0) {
+          dfb_config->dma = true;
+     } else
+     if (strcmp (name, "no-dma" ) == 0) {
+          dfb_config->dma = false;
+     } else
      if (strcmp (name, "mmx" ) == 0) {
           dfb_config->mmx = true;
      } else
      if (strcmp (name, "no-mmx" ) == 0) {
           dfb_config->mmx = false;
      } else
+     if (strcmp (name, "agp" ) == 0) {
+          if (value) {
+               int mode;
+
+               if (sscanf( value, "%d", &mode ) < 1 || mode < 0 || mode > 8) {
+                    D_ERROR( "DirectFB/Config 'agp': "
+                             "invalid agp mode '%s'!\n", value );
+                    return DFB_INVARG;
+               }
+
+               dfb_config->agp = mode;
+          }
+          else {
+               dfb_config->agp = 8; /* maximum possible */
+          }
+     } else
+     if (strcmp (name, "no-agp" ) == 0) {
+          dfb_config->agp = 0;
+     } else
+     if (strcmp (name, "agpmem-limit" ) == 0) { 
+          if (value) {
+               int limit;
+
+               if (sscanf( value, "%d", &limit ) < 1) {
+                    D_ERROR( "DirectFB/Config 'agpmem-limit': "
+                             "Could not parse value!\n" );
+                    return DFB_INVARG;
+               }
+
+               if (limit < 0)
+                    limit = 0;
+
+               dfb_config->agpmem_limit = limit << 10;
+          }
+          else {
+               D_ERROR("DirectFB/Config 'agpmem-limit': No value specified!\n");
+               return DFB_INVARG;
+          }
+     } else
      if (strcmp (name, "vt" ) == 0) {
           dfb_config->vt = true;
      } else
@@ -980,6 +1050,34 @@ DFBResult dfb_config_set( const char *name, const char *value )
                D_ERROR( "DirectFB/Config: No MuTouch device specified!\n" );
                return DFB_INVARG;
           }
+     } else
+     if (strcmp (name, "penmount-device" ) == 0) {
+          if (value) {
+               if (dfb_config->penmount_device)
+                    D_FREE( dfb_config->penmount_device );
+
+               dfb_config->penmount_device = D_STRDUP( value );
+          }
+          else {
+               D_ERROR( "DirectFB/Config: No PenMount device specified!\n" );
+               return DFB_INVARG;
+          }
+     } else
+     if (strcmp (name, "unichrome-revision" ) == 0) {
+          if (value) {
+               int rev;
+
+               if (sscanf( value, "%d", &rev ) < 1) {
+                    D_ERROR("DirectFB/Config 'unichrome-revision': Could not parse revision!\n");
+                    return DFB_INVARG;
+               }
+
+               dfb_config->unichrome_revision = rev;
+          }
+          else {
+               D_ERROR("DirectFB/Config 'unichrome-revision': No revision specified!\n");
+               return DFB_INVARG;
+          }
      }
      else
           return DFB_UNSUPPORTED;
@@ -1130,7 +1228,14 @@ DFBResult dfb_config_read( const char *filename )
 
      while (fgets( line, 400, f )) {
           char *name = line;
-          char *value = strchr( line, '=' );
+          char *comment = strchr( line, '#');
+          char *value;
+          
+          if (comment) {
+               *comment = 0;
+          }
+          
+          value = strchr( line, '=' );
 
           if (value) {
                *value++ = 0;
