@@ -138,6 +138,17 @@ typedef struct
      char      *h3600_device;                     /* H3600 Touchscreen Device */
 
      char      *mut_device;                       /* MuTouch Device */
+
+     char      *penmount_device;                  /* PenMount Device */
+
+     int        unichrome_revision;               /* Unichrome hardware
+                                                     revision number override */
+     
+     bool       dma;                              /* Enable DMA */
+     
+     int        agp;                              /* AGP mode */
+     int        agpmem_limit;                     /* Limit of AGP memory
+                                                     used by DirectFB */
 } DFBConfig;
 
 extern DFBConfig *dfb_config;
