@@ -83,15 +83,15 @@ system_initialize( CoreDFB *core, void **data )
      dfb_osx = (DFBOSX*) SHCALLOC( 1, sizeof(DFBOSX) );
      if (!dfb_osx) {
           D_ERROR( "DirectFB/OSX: Couldn't allocate shared memory!\n" );
-          return DFB_NOSYSTEMMEMORY;
+          return D_OOSHM();
      }
 
      dfb_osx_core = core;
 
      /* Initialize OSX */
-     fusion_skirmish_init( &dfb_osx->lock, "OSX System" );
+     fusion_skirmish_init( &dfb_osx->lock, "OSX System", dfb_core_world(core) );
 
-     fusion_call_init( &dfb_osx->call, dfb_osx_call_handler, NULL );
+     fusion_call_init( &dfb_osx->call, dfb_osx_call_handler, NULL, dfb_core_world(core) );
 
      screen = dfb_screens_register( NULL, NULL, &osxPrimaryScreenFuncs );
 
@@ -229,6 +229,24 @@ system_videoram_length()
      return 0;
 }
 
+static unsigned long
+system_aux_memory_physical( unsigned int offset )
+{
+     return 0;
+}
+
+static void *
+system_aux_memory_virtual( unsigned int offset )
+{
+     return NULL;
+}
+
+static unsigned int
+system_auxram_length()
+{
+     return 0;
+}
+
 static void
 system_get_busid( int *ret_bus, int *ret_dev, int *ret_func )
 {
