@@ -31,6 +31,7 @@
 
 #include <directfb.h>
 
+#include <fusion/fusion.h>
 #include <fusion/shmalloc.h>
 
 #include <core/core.h>
@@ -44,15 +45,21 @@
 #include <gfx/convert.h>
 
 #include <misc/conf.h>
+#include <misc/util.h>
 
+#include <direct/debug.h>
 #include <direct/memcpy.h>
 #include <direct/messages.h>
+#include <direct/thread.h>
 
 #include <SDL.h>
 
 #include "sdl.h"
 #include "primary.h"
 
+D_DEBUG_DOMAIN( SDL_Updates, "SDL/Updates", "SDL System Screen Updates" );
+
+
 extern DFBSDL  *dfb_sdl;
 extern CoreDFB *dfb_sdl_core;
 
@@ -62,8 +69,7 @@ static DFBResult dfb_sdl_set_video_mode( CoreDFB *core, CoreLayerRegionConfig *c
 static DFBResult dfb_sdl_update_screen( CoreDFB *core, DFBRegion *region );
 static DFBResult dfb_sdl_set_palette( CorePalette *palette );
 
-static DFBResult update_screen( CoreSurface *surface,
-                                int x, int y, int w, int h );
+static DFBResult update_screen( int x, int y, int w, int h );
 
 static SDL_Surface *screen = NULL;
 
@@ -121,6 +127,10 @@ ScreenFuncs sdlPrimaryScreenFuncs = {
 
 /******************************************************************************/
 
+static void * ScreenUpdateLoop( DirectThread *thread, void *arg );
+
+/******************************************************************************/
+
 static int
 primaryLayerDataSize()
 {
@@ -171,6 +181,15 @@ primaryInitLayer( CoreLayer                  *layer,
      else
           config->pixelformat = DSPF_RGB16;
 
+     /* Initialize update lock and condition. */
+     pthread_mutex_init( &dfb_sdl->update.lock, NULL );
+     pthread_cond_init( &dfb_sdl->update.cond, NULL );
+
+     /* Start update thread. */
+     dfb_sdl->update.thread = direct_thread_create( DTT_OUTPUT, ScreenUpdateLoop, NULL, "Screen Update" );
+     if (!dfb_sdl->update.thread)
+          return DFB_FAILURE;
+
      return DFB_OK;
 }
 
@@ -232,8 +251,12 @@ primarySetRegion( CoreLayer                  *layer,
      if (ret)
           return ret;
 
-     if (surface)
+     if (surface) {
+          pthread_mutex_lock( &dfb_sdl->update.lock );
           dfb_sdl->primary = surface;
+          dfb_sdl->update.pending = false;
+          pthread_mutex_unlock( &dfb_sdl->update.lock );
+     }
 
      if (palette)
           dfb_sdl_set_palette( palette );
@@ -247,8 +270,22 @@ primaryRemoveRegion( CoreLayer             *layer,
                      void                  *layer_data,
                      void                  *region_data )
 {
+     D_DEBUG_AT( SDL_Updates, "%s( %p )\n", __FUNCTION__, layer );
+
+     D_DEBUG_AT( SDL_Updates, "  -> locking sdl lock...\n" );
+
+     fusion_skirmish_prevail( &dfb_sdl->lock );
+
+     D_DEBUG_AT( SDL_Updates, "  -> setting primary to NULL...\n" );
+
      dfb_sdl->primary = NULL;
 
+     D_DEBUG_AT( SDL_Updates, "  -> unlocking sdl lock...\n" );
+
+     fusion_skirmish_dismiss( &dfb_sdl->lock );
+
+     D_DEBUG_AT( SDL_Updates, "  -> done.\n" );
+
      return DFB_OK;
 }
 
@@ -295,7 +332,7 @@ primaryAllocateSurface( CoreLayer              *layer,
      if (config->buffermode != DLBM_FRONTONLY)
           caps |= DSCAPS_DOUBLE;
 
-     return dfb_surface_create( NULL, config->width, config->height,
+     return dfb_surface_create( dfb_sdl_core, config->width, config->height,
                                 config->format, CSP_SYSTEMONLY,
                                 caps, NULL, ret_surface );
 }
@@ -336,7 +373,7 @@ primaryReallocateSurface( CoreLayer             *layer,
      if (ret)
           return ret;
 
-     ret = dfb_surface_reformat( NULL, surface, config->width,
+     ret = dfb_surface_reformat( dfb_sdl_core, surface, config->width,
                                  config->height, config->format );
      if (ret)
           return ret;
@@ -346,7 +383,7 @@ primaryReallocateSurface( CoreLayer             *layer,
           DFBResult    ret;
           CorePalette *palette;
 
-          ret = dfb_palette_create( NULL,    /* FIXME */
+          ret = dfb_palette_create( dfb_sdl_core,
                                     1 << DFB_COLOR_BITS_PER_PIXEL( config->format ),
                                     &palette );
           if (ret)
@@ -382,27 +419,49 @@ DisplayLayerFuncs sdlPrimaryLayerFuncs = {
 /******************************************************************************/
 
 static DFBResult
-update_screen( CoreSurface *surface, int x, int y, int w, int h )
+update_screen( int x, int y, int w, int h )
 {
      int          i;
      void        *dst;
      void        *src;
      int          pitch;
      DFBResult    ret;
+     CoreSurface *surface;
 
-     D_ASSERT( surface != NULL );
+     D_DEBUG_AT( SDL_Updates, "%s( %p, %d, %d, %d, %d )\n", __FUNCTION__, surface, x, y, w, h );
+
+     D_DEBUG_AT( SDL_Updates, "  -> locking sdl lock...\n" );
+
+     fusion_skirmish_prevail( &dfb_sdl->lock );
+
+     surface = dfb_sdl->primary;
+
+     D_DEBUG_AT( SDL_Updates, "  -> primary is %p\n", surface );
+
+     if (!surface) {
+          D_DEBUG_AT( SDL_Updates, "  -> unlocking sdl lock...\n" );
+          fusion_skirmish_dismiss( &dfb_sdl->lock );
+          D_DEBUG_AT( SDL_Updates, "  -> done.\n" );
+          return DFB_OK;
+     }
+
+     D_DEBUG_AT( SDL_Updates, "  -> locking sdl surface...\n" );
 
      if (SDL_LockSurface( screen ) < 0) {
           D_ERROR( "DirectFB/SDL: "
                    "Couldn't lock the display surface: %s\n", SDL_GetError() );
+          fusion_skirmish_dismiss( &dfb_sdl->lock );
           return DFB_FAILURE;
      }
 
+     D_DEBUG_AT( SDL_Updates, "  -> locking dfb surface...\n" );
+
      ret = dfb_surface_soft_lock( surface, DSLF_READ, &src, &pitch, true );
      if (ret) {
           D_ERROR( "DirectFB/SDL: Couldn't lock layer surface: %s\n",
                    DirectFBErrorString( ret ) );
           SDL_UnlockSurface(screen);
+          fusion_skirmish_dismiss( &dfb_sdl->lock );
           return ret;
      }
 
@@ -411,6 +470,8 @@ update_screen( CoreSurface *surface, int x, int y, int w, int h )
      src += DFB_BYTES_PER_LINE( surface->format, x ) + y * pitch;
      dst += DFB_BYTES_PER_LINE( surface->format, x ) + y * screen->pitch;
 
+     D_DEBUG_AT( SDL_Updates, "  -> copying pixels...\n" );
+
      for (i=0; i<h; ++i) {
           direct_memcpy( dst, src, DFB_BYTES_PER_LINE( surface->format, w ) );
 
@@ -418,15 +479,61 @@ update_screen( CoreSurface *surface, int x, int y, int w, int h )
           dst += screen->pitch;
      }
 
+     D_DEBUG_AT( SDL_Updates, "  -> unlocking dfb surface...\n" );
+
      dfb_surface_unlock( surface, true );
 
+     D_DEBUG_AT( SDL_Updates, "  -> unlocking sdl surface...\n" );
+
      SDL_UnlockSurface( screen );
 
+     D_DEBUG_AT( SDL_Updates, "  -> calling SDL_UpdateRect()...\n" );
+
      SDL_UpdateRect( screen, x, y, w, h );
 
+     D_DEBUG_AT( SDL_Updates, "  -> unlocking sdl lock...\n" );
+
+     fusion_skirmish_dismiss( &dfb_sdl->lock );
+
+     D_DEBUG_AT( SDL_Updates, "  -> done.\n" );
+
      return DFB_OK;
 }
 
+static void *
+ScreenUpdateLoop( DirectThread *thread, void *arg )
+{
+     pthread_mutex_lock( &dfb_sdl->update.lock );
+
+     D_DEBUG_AT( SDL_Updates, "Entering %s()...\n", __FUNCTION__ );
+
+     while (!dfb_sdl->update.quit) {
+          if (dfb_sdl->update.pending) {
+               DFBRectangle update = DFB_RECTANGLE_INIT_FROM_REGION( &dfb_sdl->update.region );
+
+               dfb_sdl->update.pending = false;
+
+               D_DEBUG_AT( SDL_Updates, "Got update %d,%d - %dx%d...\n", DFB_RECTANGLE_VALS( &update ) );
+
+               pthread_mutex_unlock( &dfb_sdl->update.lock );
+
+
+               update_screen( update.x, update.y, update.w, update.h );
+
+
+               pthread_mutex_lock( &dfb_sdl->update.lock );
+          }
+          else
+               pthread_cond_wait( &dfb_sdl->update.cond, &dfb_sdl->update.lock );
+     }
+
+     D_DEBUG_AT( SDL_Updates, "Returning from %s()...\n", __FUNCTION__ );
+
+     pthread_mutex_unlock( &dfb_sdl->update.lock );
+
+     return NULL;
+}
+
 /******************************************************************************/
 
 typedef enum {
@@ -455,6 +562,7 @@ dfb_sdl_set_video_mode_handler( CoreLayerRegionConfig *config )
      }
      /* Hide SDL's cursor */
      SDL_ShowCursor(SDL_DISABLE);
+
      fusion_skirmish_dismiss( &dfb_sdl->lock );
 
      return DFB_OK;
@@ -463,20 +571,33 @@ dfb_sdl_set_video_mode_handler( CoreLayerRegionConfig *config )
 static DFBResult
 dfb_sdl_update_screen_handler( const DFBRegion *region )
 {
-     DFBResult    ret;
+     DFBRegion    update;
      CoreSurface *surface = dfb_sdl->primary;
 
-     fusion_skirmish_prevail( &dfb_sdl->lock );
+     DFB_REGION_ASSERT_IF( region );
 
-     if (!region)
-          ret = update_screen( surface, 0, 0, surface->width, surface->height );
-     else
-          ret = update_screen( surface,
-                               region->x1,  region->y1,
-                               region->x2 - region->x1 + 1,
-                               region->y2 - region->y1 + 1 );
+     if (region)
+          update = *region;
+     else {
+          update.x1 = 0;
+          update.y1 = 0;
+          update.x2 = surface->width - 1;
+          update.y2 = surface->height - 1;
+     }
 
-     fusion_skirmish_dismiss( &dfb_sdl->lock );
+
+     pthread_mutex_lock( &dfb_sdl->update.lock );
+
+     if (dfb_sdl->update.pending)
+          dfb_region_region_union( &dfb_sdl->update.region, &update );
+     else {
+          dfb_sdl->update.region  = update;
+          dfb_sdl->update.pending = true;
+     }
+
+     pthread_cond_signal( &dfb_sdl->update.cond );
+
+     pthread_mutex_unlock( &dfb_sdl->update.lock );
 
      return DFB_OK;
 }
@@ -513,7 +634,9 @@ dfb_sdl_call_handler( int   caller,
                return dfb_sdl_set_video_mode_handler( call_ptr );
 
           case SDL_UPDATE_SCREEN:
-               return dfb_sdl_update_screen_handler( call_ptr );
+               dfb_sdl_update_screen_handler( call_ptr );
+               SHFREE( dfb_core_shmpool(dfb_sdl_core), call_ptr );
+               return 0;
 
           case SDL_SET_PALETTE:
                return dfb_sdl_set_palette_handler( call_ptr );
@@ -537,19 +660,19 @@ dfb_sdl_set_video_mode( CoreDFB *core, CoreLayerRegionConfig *config )
      if (dfb_core_is_master( core ))
           return dfb_sdl_set_video_mode_handler( config );
 
-     if (!fusion_is_shared( config )) {
-          tmp = SHMALLOC( sizeof(CoreLayerRegionConfig) );
+     if (!fusion_is_shared( dfb_core_world(core), config )) {
+          tmp = SHMALLOC( dfb_core_shmpool(core), sizeof(CoreLayerRegionConfig) );
           if (!tmp)
-               return DFB_NOSYSTEMMEMORY;
+               return D_OOSHM();
 
           direct_memcpy( tmp, config, sizeof(CoreLayerRegionConfig) );
      }
 
-     fusion_call_execute( &dfb_sdl->call, SDL_SET_VIDEO_MODE,
+     fusion_call_execute( &dfb_sdl->call, FCEF_NONE, SDL_SET_VIDEO_MODE,
                           tmp ? tmp : config, &ret );
 
      if (tmp)
-          SHFREE( tmp );
+          SHFREE( dfb_core_shmpool(core), tmp );
 
      return ret;
 }
@@ -564,22 +687,17 @@ dfb_sdl_update_screen( CoreDFB *core, DFBRegion *region )
           return dfb_sdl_update_screen_handler( region );
 
      if (region) {
-          if (!fusion_is_shared( region )) {
-               tmp = SHMALLOC( sizeof(DFBRegion) );
-               if (!tmp)
-                    return DFB_NOSYSTEMMEMORY;
+          tmp = SHMALLOC( dfb_core_shmpool(core), sizeof(DFBRegion) );
+          if (!tmp)
+               return D_OOSHM();
 
-               direct_memcpy( tmp, region, sizeof(DFBRegion) );
-          }
+          direct_memcpy( tmp, region, sizeof(DFBRegion) );
      }
 
-     fusion_call_execute( &dfb_sdl->call, SDL_UPDATE_SCREEN,
+     fusion_call_execute( &dfb_sdl->call, FCEF_ONEWAY, SDL_UPDATE_SCREEN,
                           tmp ? tmp : region, &ret );
 
-     if (tmp)
-          SHFREE( tmp );
-
-     return ret;
+     return DFB_OK;
 }
 
 static DFBResult
@@ -587,7 +705,7 @@ dfb_sdl_set_palette( CorePalette *palette )
 {
      int ret;
 
-     fusion_call_execute( &dfb_sdl->call, SDL_SET_PALETTE,
+     fusion_call_execute( &dfb_sdl->call, FCEF_NONE, SDL_SET_PALETTE,
                           palette, &ret );
 
      return ret;
