@@ -49,6 +49,7 @@
 #include <misc/conf.h>
 
 #include <direct/messages.h>
+#include <direct/thread.h>
 
 #include <SDL.h>
 
@@ -79,10 +80,10 @@ system_initialize( CoreDFB *core, void **data )
 
      D_ASSERT( dfb_sdl == NULL );
 
-     dfb_sdl = (DFBSDL*) SHCALLOC( 1, sizeof(DFBSDL) );
+     dfb_sdl = (DFBSDL*) SHCALLOC( dfb_core_shmpool(core), 1, sizeof(DFBSDL) );
      if (!dfb_sdl) {
           D_ERROR( "DirectFB/SDL: Couldn't allocate shared memory!\n" );
-          return DFB_NOSYSTEMMEMORY;
+          return D_OOSHM();
      }
 
      driver = getenv( "SDL_VIDEODRIVER" );
@@ -95,7 +96,7 @@ system_initialize( CoreDFB *core, void **data )
      if ( SDL_Init(SDL_INIT_VIDEO) < 0 ) {
           D_ERROR( "DirectFB/SDL: Couldn't initialize SDL: %s\n", SDL_GetError() );
 
-          SHFREE( dfb_sdl );
+          SHFREE( dfb_core_shmpool(core), dfb_sdl );
           dfb_sdl = NULL;
 
           return DFB_INIT;
@@ -103,9 +104,9 @@ system_initialize( CoreDFB *core, void **data )
 
      dfb_sdl_core = core;
 
-     fusion_skirmish_init( &dfb_sdl->lock, "SDL System" );
+     fusion_skirmish_init( &dfb_sdl->lock, "SDL System", dfb_core_world(core) );
 
-     fusion_call_init( &dfb_sdl->call, dfb_sdl_call_handler, NULL );
+     fusion_call_init( &dfb_sdl->call, dfb_sdl_call_handler, NULL, dfb_core_world(core) );
 
      screen = dfb_screens_register( NULL, NULL, &sdlPrimaryScreenFuncs );
 
@@ -145,6 +146,20 @@ system_shutdown( bool emergency )
 {
      D_ASSERT( dfb_sdl != NULL );
 
+     /* Stop update thread. */
+     if (dfb_sdl->update.thread) {
+          if (!emergency) {
+               dfb_sdl->update.quit = true;
+
+               pthread_cond_signal( &dfb_sdl->update.cond );
+
+               direct_thread_join( dfb_sdl->update.thread );
+          }
+
+          direct_thread_destroy( dfb_sdl->update.thread );
+     }
+
+
      fusion_call_destroy( &dfb_sdl->call );
 
      fusion_skirmish_prevail( &dfb_sdl->lock );
@@ -153,7 +168,7 @@ system_shutdown( bool emergency )
 
      fusion_skirmish_destroy( &dfb_sdl->lock );
 
-     SHFREE( dfb_sdl );
+     SHFREE( dfb_core_shmpool(dfb_sdl_core), dfb_sdl );
      dfb_sdl = NULL;
      dfb_sdl_core = NULL;
 
@@ -245,6 +260,24 @@ system_videoram_length()
      return 0;
 }
 
+static unsigned long
+system_aux_memory_physical( unsigned int offset )
+{
+     return 0;
+}
+
+static void *
+system_aux_memory_virtual( unsigned int offset )
+{
+     return NULL;
+}
+
+static unsigned int
+system_auxram_length()
+{
+     return 0;
+}
+
 static void
 system_get_busid( int *ret_bus, int *ret_dev, int *ret_func )
 {
