@@ -34,9 +34,10 @@
 #                                                              #
 ################################################################
 
+$version = shift @ARGV;
 
-html_create( INDEX, "index.html", "Index Page" );
-html_create( TYPES, "types.html", "DirectFB Types" );
+html_create( INDEX, "index.html", "Index Page", "", "Index" );
+html_create( TYPES, "types.html", "DirectFB Types", "", "Types" );
 
 print INDEX "<P>\n",
             "  <CENTER>\n",
@@ -254,7 +255,7 @@ sub parse_interface (NAME)
       html_create( INTERFACE, "$interface.html",
                               "<A href=\"index.html\">" .
                               "<FONT color=#DDDDDD>DirectFB Interfaces</FONT>" .
-                              "</A>", $interface );
+                              "</A>", $interface, $interface );
 
 #      print INTERFACE "<P align=center>\n",
 #                      "  $interface_abstracts{$interface}\n",
@@ -294,7 +295,7 @@ sub parse_interface (NAME)
                   html_create( FUNCTION, "${interface}_$1.html",
                                "<A href=\"$interface.html\">" .
                                "<FONT color=#DDDDDD>$interface</FONT>" .
-                               "</A>", $1 );
+                               "</A>", $1, "$interface - $1" );
 
                   print FUNCTION "<P>$headline</P><P>\n",
                                  "  <TABLE border=0 cellspacing=0 cellpadding=2 bgcolor=#C0C0C0>\n",
@@ -821,12 +822,13 @@ sub parse_macro (NAME, PARAMS)
    }
 
 
-sub html_create (FILEHANDLE, FILENAME, TITLE, SUBTITLE)
+sub html_create (FILEHANDLE, FILENAME, TITLE, SUBTITLE, SINGLETITLE)
    {
       my $FILE = shift(@_);
       my $filename = shift(@_);
       my $title = shift(@_);
       my $subtitle = shift(@_);
+      my $singletitle = shift(@_);
 
       open( $FILE, ">$filename" )
           or die ("*** Can not open '$filename' for writing:\n*** $!");
@@ -841,7 +843,7 @@ sub html_create (FILEHANDLE, FILENAME, TITLE, SUBTITLE)
                   "  A:link, A:visited, A:active { text-decoration: none; }\n",
                   "</STYLE>\n",
                   "<HEAD>\n",
-                  "  <TITLE>DirectFB Reference Manual</TITLE>\n",
+                  "  <TITLE>$singletitle [DirectFB Reference Manual]</TITLE>\n",
                   "</HEAD>\n",
                   "<BODY bgcolor=#FFFFFF link=#0070FF",
                        " vlink=#0070FF text=#404040>\n",
@@ -851,7 +853,7 @@ sub html_create (FILEHANDLE, FILENAME, TITLE, SUBTITLE)
                   "    <A href=\"http://www.directfb.org\"><IMG border=0 src=\"directfb.png\"></A>\n",
                   "  </TD><TD align=right>\n",
                   "    &nbsp;&nbsp;",
-                  "    <A href=\"index.html\"><FONT size=+3 color=white>DirectFB Reference Manual</FONT></A>\n",
+                  "    <A href=\"index.html\"><FONT size=+3 color=white>Reference Manual - $version</FONT></A>\n",
                   "  </TD></TR>\n",
                   "  <TR><TD colspan=2 align=center bgcolor=#303030>\n";
 
